%% FORWARD GUIDANCE STRUCTURAL VAR IDENTIFIED WITH SIGN-RESRICTIONS AND FORECAST CONSISTENT PRIOR
%% DOH AND SMITH, 2021

clear;
clear legend;
rng(123456,'twister');

%% ADD VAR AUXILIARY FILES
cwd = strsplit(pwd, '\');
cwd = strjoin(cwd(1:end-1), '\');
cwd = [cwd, '\VARAuxiliaryFiles'];
addpath(cwd);

% SET STRING BASE TO SAVE FIGURES
save_string = 'VARUhligFGFullSample';

% LOAD DATA
[uhligvar_data, uhligvar_text] = xlsread('uhligvar_data_with_labels.xlsx','Sheet1');
[bluechip_data, bluechip_text] = xlsread('bluechip_data_with_labels.xlsx','BC_2016-02-10');

% DEFINE AND TRANSFORM UHLIG VAR DATA (IF NEEDED)
uhligvar_dates  = uhligvar_text(10:end,1);
uhligvar_vnames = uhligvar_text(9,2:end);
Y1(:,1) = log(uhligvar_data(:,3))*100;		% Real GDP
Y1(:,2) = log(uhligvar_data(:,7))*100;		% CPI
Y1(:,3) = log(uhligvar_data(:,8))*100;		% Commodity Price Index
Y1(:,4) = uhligvar_data(:,9);				% Fed Funds Rate
Y1(:,5) = log(uhligvar_data(:,12))*100;		% Non-Borrowed Reserves
Y1(:,6) = log(uhligvar_data(:,11))*100;		% Total Reserves

Y1varnames = [uhligvar_vnames(3); uhligvar_vnames(7); uhligvar_vnames(8); uhligvar_vnames(9); uhligvar_vnames(12); uhligvar_vnames(11)]

% DEFINE BLUECHIP DATA
bluechip_dates  = bluechip_text(2:end,1);
bluechip_vnames = bluechip_text(1,2:end);

tbill6  = bluechip_data(:,find(strcmp(bluechip_vnames,'tbill6')));		% 4-quarter ahead 3-month T-bill BlueChip Forecast

%% SET LAG LENGTH
vlag = 3;

%% CHOOSE VAR ESTIMATION SAMPLE
start_date  = '1994-Jan';
end_date    = '2015-Dec';	

uhligvar_start	= find(strcmp(uhligvar_dates, start_date)) - vlag;
uhligvar_end 	= find(strcmp(uhligvar_dates, end_date));

bluechip_start	= find(strcmp(bluechip_dates, start_date)) - vlag;
bluechip_end 	= find(strcmp(bluechip_dates, end_date));

Y 				= [Y1(uhligvar_start:uhligvar_end,:), tbill6(bluechip_start:bluechip_end,1)];

%% SPECIFY LOCATION OF SURVEY FORECAST AND REALIZED DATA
s_loc	= 7;	% Location of survey measure in VAR
d_loc	= 4;	% Location of data measure in VAR

%% ESTIMATE WITH CONSTANT
iscon = 1;

%% NOW ESTIMATE
[Bcomp, cvec, Bpl_ev, VC_eps, Residmat] = ...
									estim(Y, vlag, iscon);
									
%% SET NUMBER OF IRF PERIODS (adds first period so 15 implies 16 periods will be computed)
IRhoriz = 47+(12+1);

%% SET OPTIONS FOR SIGN RESTRICTIONS
% Restrictions: positive 1, negative -1, unrestricted 0
R(:,:,1) = [ 0     0     0    % Real GDP
             1     6    -1    % Inflation
             1     6    -1    % Commodity Price
             0     0     0    % Fed Fund
             0     0     0    % NonBorr. Reserves
             0     0     0    % Total Reserves
			 1     6     1];  % Blue Chip Forecast of 4-Quarter Tbill Rate

%% SPECIFY VAR PRIOR
% Uninformative priors
[Tbig, Nbig] = size(Y);
A_prior = zeros(iscon+Nbig*vlag,Nbig);   
a_prior = A_prior(:);
V_prior = 1e12*eye(iscon+Nbig*vlag);
% Hyperparameters on inv(SIGMA) ~ W(v_prior,inv(S_prior))
v_prior = 0;
scale1  = Tbig-vlag-1;	
S_prior = 0*scale1*eye(Nbig);

%% SET OPTIONS FOR POSTERIOR SIMULATION
NMCDraws	  = 0;
NUhligDraws   = 200000;
NTargetAccept = 5000;
CIperc		  = [0.16,0.84];
[allIRs, allSResids] = MCMCUhligDraws(Bcomp,VC_eps,cvec,R,IRhoriz,CIperc,Y,NMCDraws,NUhligDraws,NTargetAccept,A_prior,a_prior,V_prior,v_prior,S_prior);

%% CALIBRATE LAMBDA & RE-WEIGHT POSTERIOR DRAWS ACCORDING TO FORECAST CONSISTENT PRIOR	
lambda_ml = 8.03e8;

lambda = lambda_ml;
[allIRs_im, allSResids_im, mm_best, mm_worst, index_im, effective_sample] = WeakConsistencyPrior(allIRs,allSResids,lambda,s_loc,d_loc);

%% PLOT & SAVE IRFS
%plotIRF1(allIRs,CIperc,s_loc,d_loc,0,save_string)

%save_string_im = [save_string,'IM'];
%plotIRF1(allIRs_im,CIperc,s_loc,d_loc,lambda,save_string_im)

save_string_overlay = [save_string, 'IMOverlay'];
plotIRF3(allIRs,allIRs_im,CIperc,s_loc,d_loc,lambda,save_string_overlay)

