%% Posterior IRFs After Imposing Weak Consistency Prior
%% For Forward Guidance Application in
%% "Reconciling VAR-based Forecasts with Survey Foreccasts"

function [allIRs_im, allSResids_im, mm_best, mm_worst, index_im, effective_sample] = WeakConsistencyPrior(allIRs,allSResids,lambda,s_loc,d_loc);

[IRhoriz, Nbig, Nbig, NAccept] = size(allIRs);
IRhoriz = IRhoriz;
Nshocks = 1;

for mm = 1:1:NAccept;
	for nn = 1:1:Nshocks;
		
		shock_size =  var(squeeze(allIRs(1,s_loc,nn,:)))^0.5;
		
		for hh = 13:1:IRhoriz-1;
			gA(hh,1) = (mean(allIRs(hh-1:hh+1,d_loc,nn,mm)) - allIRs(hh-12,s_loc,nn,mm))/100;
		end;
		
		p_pdf 		 = -0.5*lambda*mean(diag(gA*gA'));
		allp_pdf(mm) = ((shock_size/allIRs(1,s_loc,nn,mm))^2)*p_pdf;
		
	end;
end;		

[weight_best,mm_best]   = max(allp_pdf);
[weight_worst,mm_worst] = min(allp_pdf);

%% Construct Importance Sampling Weights
w_im 	 = exp(allp_pdf-max(allp_pdf))./sum(exp(allp_pdf-max(allp_pdf)));
index_im = randsample(1:NAccept,NAccept,true,w_im'); 
index_im = index_im';

%% Report Sample Size
sample = NAccept;
effective_sample = 1/(sum(w_im.^2));

%% Calculate Reweighted IRFs
allIRs_im = allIRs(:,:,:,index_im);
allSResids_im = allSResids(:,:,index_im);