%% THIS FUNCTION PLOTS THE MEDIAN IRF AND CI FOR THE RE-SAMPLED DRAWS
%% AS WELL AS THE MEDIAN IRF OF THE ORIGINAL DRAWS

function plotIRF3(allIRs,allIRs_im,CIperc,s_loc,d_loc,lambda,save_string)
clear legend;

%% DEFINE PERIOD TO PLOT
IRhoriz = size(allIRs,1);	% Length of IRF
irflength = IRhoriz-13;		% Length of IRF to plot
t = 1:irflength;

%% GET LOWER, MEDIAN AND UPPER VARIABLES OF INTEREST FROM SIMULATED DISTRIBUTION
%% IRFS
for hh=1:size(allIRs_im,1);
    for ii=1:size(allIRs_im,2);
        for jj=1:size(allIRs_im,3);
            sliceloc   = squeeze(allIRs_im(hh,ii,jj,:));
            IRpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            IRlo_im(hh,ii,jj)   = IRpercsloc(1);
			IRmed_im(hh,ii,jj)  = IRpercsloc(2);
            IRup_im(hh,ii,jj)   = IRpercsloc(3);
        end
    end
end

for hh=1:size(allIRs,1);
    for ii=1:size(allIRs,2);
        for jj=1:size(allIRs,3);
            sliceloc   = squeeze(allIRs(hh,ii,jj,:));
            IRpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            IRlo(hh,ii,jj)   = IRpercsloc(1);
			IRmed(hh,ii,jj)  = IRpercsloc(2);
            IRup(hh,ii,jj)   = IRpercsloc(3);
        end
    end
end

var1y	 	    = NaN(irflength,1);
var1y_im 	    = NaN(irflength,1);
for hh = 13:1:IRhoriz-1;
	var1y(hh-12)   	   = mean(IRmed(hh-1:hh+1,d_loc,1));
	var1y_im(hh-12)    = mean(IRmed_im(hh-1:hh+1,d_loc,1));
end;

IRdev  	 = var1y  - IRmed(1:size(var1y),s_loc,1);
IRdev_im = var1y_im - IRmed_im(1:size(var1y_im),s_loc,1);

dev		= -cumsum(IRdev);
dev_im	= -cumsum(IRdev_im);

CumulativeForecastDeviationLambda0	 = dev(end)
CumulativeForecastDeviationLambdaPos = dev_im(end)
CumulativeForecastDeviationReduction = CumulativeForecastDeviationLambda0 - CumulativeForecastDeviationLambdaPos

figure
subplot(4,2,1)
hold on
jbfill(t,-IRup_im(1:irflength,1,1)',-IRlo_im(1:irflength,1,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,1,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,1,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Output','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.1 0.4]);set(gca,'YTick',[-0.1:0.1:0.4],'FontSize',12);
ylabel('Percent','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(4,2,2)
hold on
jbfill(t,-IRup_im(1:irflength,2,1)',-IRlo_im(1:irflength,2,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,2,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,2,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('CPI','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.25 1.25]);set(gca,'YTick',[-0.25:0.25:1.25],'FontSize',12);
ylabel('Percent','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(4,2,3)
hold on
jbfill(t,-IRup_im(1:irflength,3,1)',-IRlo_im(1:irflength,3,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,3,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,3,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Commodity Prices','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.2 0.6]);set(gca,'YTick',[-0.2:0.2:0.6],'FontSize',12);
ylabel('Percent','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(4,2,4)
hold on
jbfill(t,-IRup_im(1:irflength,4,1)',-IRlo_im(1:irflength,4,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,4,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,4,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Fed Funds','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.1 0.4]);set(gca,'YTick',[-0.1:0.1:0.4],'FontSize',12);
ylabel('APR','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(4,2,5)
hold on
jbfill(t,-IRup_im(1:irflength,5,1)',-IRlo_im(1:irflength,5,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,5,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,5,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1);  
title('Non-Borrowed Reserves','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.08 0.02]);set(gca,'YTick',[-0.08:0.02:.02],'FontSize',12);
ylabel('Percent','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(4,2,6)
hold on
jbfill(t,-IRup_im(1:irflength,6,1)',-IRlo_im(1:irflength,6,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,6,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,6,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Reserves','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.1 0.05]);set(gca,'YTick',[-0.1:0.05:0.05],'FontSize',12);
ylabel('Percent','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(4,2,7)
hold on
h1=jbfill(t,-IRup_im(1:irflength,7,1)',-IRlo_im(1:irflength,7,1)',[.8, .8, .8],[1 1 1],0,0.6);
h2=plot(t,-IRmed_im(1:irflength,7,1),'-b','LineWidth',3);
h3=plot(t,-IRmed(1:irflength,7,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Blue Chip 4-Qtr Ahead Rate','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.1 0.05]);set(gca,'YTick',[-0.1:0.05:0.05],'FontSize',12);
ylabel('APR','FontSize',12);
hleglines = [h2(1) h1(1) h3(1)];
leg = legend(hleglines, 'Impulse Response (\lambda>0)', '68% Error Band', 'Impulse Response (\lambda=0)');
set(leg,'Location','SouthEast','FontSize',11);  
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

% subplot(4,2,8)
% hold on
% plot(t,-cumsum(IRdev_im),'-b','LineWidth',3);
% plot(t,-cumsum(IRdev),'-.g','LineWidth',3);
% plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
% title('Cumulative Forecast Deviation','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
% ylim([-0.1 0.4]);set(gca,'YTick',[-0.1:0.1:0.4],'FontSize',12);
% ylabel('APR','FontSize',12);
% box on; grid on;
% set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

set(gcf, 'renderer', 'painters');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperOrientation', 'portrait');
set(gcf, 'PaperPosition', [-0.3 -0.7 9.5 12.0]);
print([save_string,'.pdf'],'-dpdf')