%% THIS FUNCTION PLOTS MEDIAN IRFS AND CI

function plotIRF5(allIRs,allIRs_im,CIperc,fig_num,col_loc,save_string,title_string,leg_on,save_on);
clear legend;

%% DEFINE PERIOD TO PLOT
IRhoriz = size(allIRs,1);	% Length of IRF
irflength = IRhoriz-13;		% Length of IRF to plot
t = 1:irflength;

%% GET LOWER, MEDIAN AND UPPER VARIABLES OF INTEREST FROM SIMULATED DISTRIBUTION
%% IRFS
for hh=1:size(allIRs_im,1);
    for ii=1:size(allIRs_im,2);
        for jj=1:size(allIRs_im,3);
            sliceloc   = squeeze(allIRs_im(hh,ii,jj,:));
            IRpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            IRlo_im(hh,ii,jj)   = IRpercsloc(1);
			IRmed_im(hh,ii,jj)  = IRpercsloc(2);
            IRup_im(hh,ii,jj)   = IRpercsloc(3);
        end
    end
end

for hh=1:size(allIRs,1);
    for ii=1:size(allIRs,2);
        for jj=1:size(allIRs,3);
            sliceloc   = squeeze(allIRs(hh,ii,jj,:));
            IRpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            IRlo(hh,ii,jj)   = IRpercsloc(1);
			IRmed(hh,ii,jj)  = IRpercsloc(2);
            IRup(hh,ii,jj)   = IRpercsloc(3);
        end
    end
end

figure(fig_num);
subplot(5,3,1+col_loc-1)
hold on
jbfill(t,-IRup_im(1:irflength,1,1)',-IRlo_im(1:irflength,1,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,1,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,1,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title({title_string;'Output'},'FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
ylim([-0.15 0.25]);set(gca,'YTick',[-0.1:0.1:0.2],'FontSize',12);
ylabel('Percent','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(5,3,4+col_loc-1)
hold on
jbfill(t,-IRup_im(1:irflength,2,1)',-IRlo_im(1:irflength,2,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,2,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,2,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('CPI','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
ylim([-0.05 0.25]);set(gca,'YTick',[0:0.1:0.2],'FontSize',12);
ylabel('Percent','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(5,3,7+col_loc-1)
hold on
jbfill(t,-IRup_im(1:irflength,3,1)',-IRlo_im(1:irflength,3,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,3,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,3,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Commodity Prices','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
ylim([-0.15 2.15]);set(gca,'YTick',[0:0.5:2.0],'FontSize',12);
ylabel('Percent','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(5,3,10+col_loc-1)
hold on
jbfill(t,-IRup_im(1:irflength,4,1)',-IRlo_im(1:irflength,4,1)',[.8, .8, .8],[1 1 1],0,0.6);
plot(t,-IRmed_im(1:irflength,4,1),'-b','LineWidth',3);
plot(t,-IRmed(1:irflength,4,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Fed Funds','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
ylim([-0.2 0.1]);set(gca,'YTick',[-0.15:0.05:0.05],'FontSize',12);
ylabel('APR','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(5,3,13+col_loc-1)
hold on
h1=jbfill(t,-IRup_im(1:irflength,7,1)',-IRlo_im(1:irflength,7,1)',[.8, .8, .8],[1 1 1],0,0.6);
h2=plot(t,-IRmed_im(1:irflength,7,1),'-b','LineWidth',3);
h3=plot(t,-IRmed(1:irflength,7,1),'-.g','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Blue Chip 4-Qtr Ahead Rate','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
ylim([-0.35 0.15]);set(gca,'YTick',[-0.30:0.1:0.1],'FontSize',12);
ylabel('APR','FontSize',12);
if leg_on == 1
	hleglines = [h2(1) h1(1) h3(1)];
	leg = legend(hleglines, 'IRF (\lambda>0)', ['68% Error Band'], 'IRF (\lambda=0)');
	set(leg,'Location','SouthEast','FontSize',10);  
end;
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

if save_on == 1;
	set(gcf, 'renderer', 'painters');
	set(gcf, 'PaperPositionMode', 'manual');
	set(gcf, 'PaperUnits', 'inches');
	set(gcf, 'PaperOrientation', 'portrait');
	set(gcf, 'PaperPosition', [-0.5 -0.7 9.5 12.0]);
	print([save_string,'.pdf'],'-dpdf')
end