%% THIS FUNCTION PLOTS THE RESPONSE OF OUTPUT 18 MONTHS AFTER A FG SHOCK (MEDIAN IRF AND CI FOR THE RE-SAMPLED DRAWS)
%% FOR VARIOUS VALUES OF LAMBDA

function plotIRFOutput18mos(lambdasp,Output18mos,lambda_ml,save_string)
clear legend;

%% PLOT 18 MONTH OUTPUT IRFS
fig_num = get(gcf,'Number');
figure(fig_num+1)
hold on
h1=jbfill(lambdasp,-Output18mos(:,1)',-Output18mos(:,3)',[.8, .8, .8],[1 1 1],0,0.6);
h2=plot(lambdasp,-Output18mos(:,2),'-b','LineWidth',3);
plot(lambdasp,zeros(length(lambdasp),1), '-k', 'LineWidth', 1); 
plot((lambda_ml/1e8)*ones(length(-0.1:0.1:0.3),1),-0.1:0.1:0.3,'--r','LineWidth',2);
text((lambda_ml+0.25e8)/1e8,0.28,{'\lambda calibrated by               ';'margnal likelihood criterion'},'Color','red','FontSize',16);
title('Impulse Response of Output at 18 Months','FontSize',20);xlim([lambdasp(1) lambdasp(end)]);
set(gca,'XTick',linspace(lambdasp(1),lambdasp(end),11));
ylim([-0.1 0.3]);set(gca,'YTick',[-0.1:0.1:0.3],'FontSize',18);
ylabel('Percent','FontSize',18);
%xlabel('\lambda','FontSize',12);
xlabel('Hyperparameter Governing Tightness of Forecast Consistent Prior: \lambda (\times 10^{8})','FontSize',18);
hleglines = [h2(1) h1(1)];
leg = legend(hleglines, 'Median Impulse Response','68% Error Band');
set(leg,'Location','NorthWest','FontSize',18);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

set(gcf, 'renderer', 'painters');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperOrientation', 'landscape');
set(gcf, 'PaperPosition', [-0.5 -0.0 11.0 6.5]);
set(gcf, 'PaperSize', [9.75 6.5]);
print([save_string,'Output18mos','.pdf'],'-dpdf')