%--------------------------------------------------------------------------
% Posterior VAR Simulation with Uhlig Style Sign Restrictions (2005, JME)
%--------------------------------------------------------------------------

function [ratio] = priordraw(Bcomp,VC_eps,R,IRhoriz,NUhligDraws,NTargetAccept);

Nbig	= length(VC_eps);
Nshocks = size(R,3);
NAccept = 0;
keep_drawingUhlig 	= 1;
mmUhlig 			= 1;
C_draw 			= chol(VC_eps)'; %% Take Cholesky Factor to Orthogonlaize Shocks

%% DO UHLIG DRAW FOR GIVEN REDUCED FORM VAR PARAMETERS
while keep_drawingUhlig == 1;
	workbar(max(NAccept,1)/NTargetAccept,'Performing Uhlig Draws w/ MCMC','Progress');
	
		%% ROTATE CHOLESKY FACTOR RANDOMLY (CALL THIS ROTATION 'FACTOR')
		% HERE WE FOLLOW KILIAN AND LUTKEPOHL (Pg. 427) INSTEAD OF UHLIG TO GENERALIZE TO MULTIPLE SHOCKS
		%[Q,R] 	= qr(reshape(mvnrnd(zeros(Nbig*Nbig,1)',eye(Nbig*Nbig)),Nbig,Nbig));
		%S		= Q*diag(sign(diag(R)));
	S = OrthNorm(Nbig);
	Factor  = C_draw*S';
		
		%% CALCULATE IRFS
	IR = VAR_irf_factor(Bcomp, Factor, IRhoriz);
		
		%% DECIDE TO KEEP OR REJECT ROTATION BASED ON SIGN RESTRICTIONS
	checkall = ones(Nbig,Nshocks);
	checkall_flip = ones(Nbig,Nshocks);
	for ss=1:Nshocks
		for ii = 1:Nbig
			if R(ii,1,ss) ~= 0
				if R(ii,3,ss) == 1
						check = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) > 0;
						checkall(ii,ss) = min(check);
						% Check flipped signs
						check_flip = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) < 0;
						checkall_flip(ii,ss) = min(check_flip);
				elseif R(ii,3,ss) == -1
						check = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) < 0;
						checkall(ii,ss) = min(check);
						% Check flipped signs
						check_flip = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) > 0;
						checkall_flip(ii,ss) = min(check_flip);
				end
			end
		end
	end

		%% IF RESTRICTIONS ARE SATISFIED, STORE IRFS
	if min(min(checkall))==1
		NAccept = NAccept + 1;
		allIRs(:,:,:,NAccept) = IR;
		inv_Factor = inv(Factor);
		allSResids(:,:,NAccept) = RFResids_draw*inv_Factor';
		workbar(NAccept/NTargetAccept,'Performing Uhlig Draws w/ MCMC','Progress');
	elseif min(min(checkall_flip))==1
		NAccept = NAccept + 1;
		allIRs(:,:,:,NAccept) = -1*IR;
		inv_Factor = inv(Factor);
		allSResids(:,:,NAccept) = -1*RFResids_draw*inv_Factor';
		workbar(NAccept/NTargetAccept,'Performing Uhlig Draws w/ MCMC','Progress');
	end
		
		%% CHECK WHETHER TO EXIT UHLIG DRAWING LOOP OR CONTINUE IN UHLIG DRAWING LOOP
	if NAccept == NTargetAccept
		keep_drawingUhlig = 0;
			disp('***** Targeted Number of Accepted Draws Reached *****');
			disp(['Percentage of Joint Draws Accepted:	' num2str(100*NAccept/(mmUhlig))]);
            ratio = NAccept/mmUhlig; 
	elseif NAccept < NTargetAccept & mmUhlig >= NUhligDraws
			keep_drawingUhlig = 0;
            ratio = NAccept/mmUhlig;
	elseif NAccept < NTargetAccept & mmUhlig < NUhligDraws
			mmUhlig = mmUhlig + 1;
	end
end
	
	