// Doh and Smith (2022) "A New Approach to Integrating Expectations Into VAR Models"
// New-Keynesian Model With MP News Shocks (Forward Guidance Shocks) & Noisy Survey Forecasts
// Alternative DGP: 1-period ahead forward guidance with no policy reaction in period t
//					Perfect Forecast Consistency for FG Shocks

// Shared Variables Block
var	a, u, n,
	fg_shock_t_1, fg_shock_t,
	a_shock, u_shock, fg_shock, n_shock,
	
// Non-Policy Shocks Block (np)
	x_np, pi_np, i_np,
	expi1_np, expi2_np, expi3_np, expi4_np, expi5_np, expi6_np, expi7_np, expi8_np, expi9_np, expi10_np, expi11_np, expi12_np,
	

// Forward Guiidance Shocks Block (fg)
	x_fg, pi_fg, i_fg,
	expi1_fg, expi2_fg, expi3_fg, expi4_fg, expi5_fg, expi6_fg, expi7_fg, expi8_fg, expi9_fg, expi10_fg, expi11_fg, expi12_fg,
	
// Observables Block
	x, pi, i, expi1;
	
varexo e_fg e_n e_a e_u;

varobs x, pi, i, expi1;

parameters beta, kappa, rho_a, sigma_a, rho_u, sigma_u,
		   phi_i, phi_x, phi_dx, phi_pi, sigma_fg,
		   rho_exp, sigma_n;

load parameterfile;
beta            = 0.99;
kappa 	        = 0.002;
rho_a			= 0.98;
sigma_a			= 0.08; 
rho_u			= 0.0;
sigma_u			= 0.002; 
phi_i			= 0.84;
phi_x	        = 0.22;
phi_dx			= 0.18;
phi_pi			= 1.42;
sigma_fg		= 0.0025;
rho_exp			= rho_exp_val;
sigma_n			= 4*sigma_fg;


model(linear);

////////////////////////////
// Shared Variables Block //
////////////////////////////
a			 = rho_a*a(-1) + sigma_a*e_a;
u			 = rho_u*u(-1) + sigma_u*e_u;
n			 = sigma_n*e_n;
fg_shock_t_1 = sigma_fg*e_fg(-1);
fg_shock_t	 = 0.25*sigma_fg*e_fg;

// DSGE Shocks
fg_shock	= sigma_fg*e_fg; // Announced at time "t", realized at "t+1"
a_shock 	= sigma_a*e_a;
u_shock		= sigma_u*e_u;
n_shock		= sigma_n*e_n;

//////////////////////////////////
// Non-Policy Shocks Block (np) //
//////////////////////////////////
// Private Sector
x_np 		= x_np(+1) - (i_np - pi_np(+1) - a + a(+1));
pi_np		= kappa*(x_np) + beta*pi_np(+1) + u;

// "Survey" Forecaster's Interest-Rate Expectations
expi1_np	= (1-rho_exp)*i_np(+1)  + rho_exp*expi1_np(-1) + n;
expi2_np	= (1-rho_exp)*i_np(+2)  + rho_exp*expi2_np(-1) + n;
expi3_np	= (1-rho_exp)*i_np(+3)  + rho_exp*expi3_np(-1) + n;
expi4_np	= (1-rho_exp)*i_np(+4)  + rho_exp*expi4_np(-1) + n;
expi5_np	= (1-rho_exp)*i_np(+5)  + rho_exp*expi5_np(-1) + n;
expi6_np	= (1-rho_exp)*i_np(+6)  + rho_exp*expi6_np(-1) + n;
expi7_np	= (1-rho_exp)*i_np(+7)  + rho_exp*expi7_np(-1) + n;
expi8_np	= (1-rho_exp)*i_np(+8)  + rho_exp*expi8_np(-1) + n;
expi9_np	= (1-rho_exp)*i_np(+9)  + rho_exp*expi9_np(-1) + n;
expi10_np	= (1-rho_exp)*i_np(+10) + rho_exp*expi10_np(-1) + n;
expi11_np	= (1-rho_exp)*i_np(+11) + rho_exp*expi11_np(-1) + n;
expi12_np	= (1-rho_exp)*i_np(+12) + rho_exp*expi12_np(-1) + n;

// Monetary Policy Rule
i_np		= phi_i*i_np(-1) + (1-phi_i)*(phi_x*x_np + phi_dx*(x_np-x_np(-1)) + phi_pi*pi_np);

////////////////////////////////////////
// Forward Guidance Shocks Block (fg) //
////////////////////////////////////////
// Private Sector
x_fg 		= x_fg(+1) - (i_fg - pi_fg(+1));
pi_fg		= kappa*(x_fg) + beta*pi_fg(+1);

// "Survey" Forecaster's Interest-Rate Expectations
expi1_fg		= i_fg(+1);
expi2_fg		= i_fg(+2);
expi3_fg		= i_fg(+3);
expi4_fg		= i_fg(+4);
expi5_fg		= i_fg(+5);
expi6_fg		= i_fg(+6);
expi7_fg		= i_fg(+7);
expi8_fg		= i_fg(+8);
expi9_fg		= i_fg(+9);
expi10_fg		= i_fg(+10);
expi11_fg		= i_fg(+11);
expi12_fg		= i_fg(+12);

// Monetary Policy Rule
i_fg			= phi_i*i_fg(-1) + (1-phi_i)*(phi_x*x_fg + phi_dx*(x_fg-x_fg(-1)) + phi_pi*pi_fg) + fg_shock_t_1 + fg_shock_t;

////////////////////////
// Observables Block //
///////////////////////
x 				= x_np + x_fg;
pi 				= pi_np + pi_fg;
i				= i_np + i_fg;
expi1			= expi1_np + expi1_fg;

end;

check;

shocks;
var e_fg = 1;
var e_n = 1;
var e_a = 1;
var e_u = 1;
end;

stoch_simul(order=1,periods=168000,irf=48,noprint,nograph);
//stoch_simul(order=1,irf=48) x pi i expi1 fg_shock;