// Doh and Smith (2022) "A New Approach to Integrating Expectations Into VAR Models"
// New-Keynesian Model With MP News Shocks (Forward Guidance Shocks) & Noisy Survey Forecasts
// Alternative DGP: 1-period ahead forward guidance with policy reaction in period t

var x, pi, i, a, u, n,
	expi1, expi2, expi3, expi4, expi5, expi6, expi7, expi8, expi9, expi10, expi11, expi12,
	fg_shock_t_1, fg_shock_t,
	a_shock, u_shock, fg_shock, n_shock;
	
varexo e_fg e_n e_a e_u;

varobs x, pi, i, expi1;

parameters beta, kappa, rho_a, sigma_a, rho_u, sigma_u,
		   phi_i, phi_x, phi_dx, phi_pi, sigma_fg,
		   rho_exp, sigma_n;

load parameterfile;
beta            = 0.99;
kappa 	        = 0.002;
rho_a			= 0.98;
sigma_a			= 0.08; 
rho_u			= 0.0;
sigma_u			= 0.002; 
phi_i			= 0.84;
phi_x	        = 0.22;
phi_dx			= 0.18;
phi_pi			= 1.42;
sigma_fg		= 0.0025;
rho_exp			= rho_exp_val;
sigma_n			= 4*sigma_fg;


model(linear);

// Private Sector
x 			= x(+1) - (i - pi(+1) - a + a(+1));
pi			= kappa*(x) + beta*pi(+1) + u;
a			= rho_a*a(-1) + sigma_a*e_a;
u			= rho_u*u(-1) + sigma_u*e_u;

// "Survey" Forecaster's Interest-Rate Expectations
expi1		= (1-rho_exp)*i(+1) + rho_exp*expi1(-1) + n;
expi2		= (1-rho_exp)*i(+2) + rho_exp*expi2(-1) + n;
expi3		= (1-rho_exp)*i(+3) + rho_exp*expi3(-1) + n;
expi4		= (1-rho_exp)*i(+4) + rho_exp*expi4(-1) + n;
expi5		= (1-rho_exp)*i(+5) + rho_exp*expi5(-1) + n;
expi6		= (1-rho_exp)*i(+6) + rho_exp*expi6(-1) + n;
expi7		= (1-rho_exp)*i(+7) + rho_exp*expi7(-1) + n;
expi8		= (1-rho_exp)*i(+8) + rho_exp*expi8(-1) + n;
expi9		= (1-rho_exp)*i(+9) + rho_exp*expi9(-1) + n;
expi10		= (1-rho_exp)*i(+10) + rho_exp*expi10(-1) + n;
expi11		= (1-rho_exp)*i(+11) + rho_exp*expi11(-1) + n;
expi12		= (1-rho_exp)*i(+12) + rho_exp*expi12(-1) + n;
n			= sigma_n*e_n;

// Monetary Policy Rule
i			= phi_i*i(-1) + (1-phi_i)*(phi_x*x + phi_dx*(x-x(-1)) + phi_pi*pi) + fg_shock_t_1 + fg_shock_t;
fg_shock_t_1 = sigma_fg*e_fg(-1);
fg_shock_t	 = 0;

// DSGE Shocks
fg_shock	= sigma_fg*e_fg; // Announced at time "t", realized at "t+1"
a_shock 	= sigma_a*e_a;
u_shock		= sigma_u*e_u;
n_shock		= sigma_n*e_n;

end;

check;

shocks;
var e_fg = 1;
var e_n = 1;
var e_a = 1;
var e_u = 1;
end;

stoch_simul(order=1,periods=168000,irf=48,noprint,nograph);
//stoch_simul(order=1,irf=48) x pi i expi1 fg_shock;