%% THIS SCRIPT REPLICATES FIGURE C.4 in the Appendix 
%% IN DOH AND SMITH (2021) "A NEW APPROACH TO INTEGRATING EXPECTATIONS INTO VAR MODELS"

clear all; 

%% RUN THE DSGE-VAR SIMULATIONS UNLESS THEY HAVE ALREADY BEEN SAVED
if isfile('SVAR_IRFS_NoFS.mat')==0
	SVAR_MC_IRFS_NoFS
else
	load 'SVAR_IRFS_NoFS.mat'
end;

if isfile('SVAR_IRFS_NoFS_4period.mat')==0
	SVAR_MC_IRFS_NoFS_4period
else
	load 'SVAR_IRFS_NoFS_4period.mat'
end;

%% LABEL IRFS TO PLOT
%% ALT DGP 1: Allowing For Policy Reaction to Forward Guidance
x_e_fg_alt1     = SVAR_IRFS_NoFS.dsge(:,1);
pi_e_fg_alt1    = SVAR_IRFS_NoFS.dsge(:,2); 
i_e_fg_alt1     = SVAR_IRFS_NoFS.dsge(:,3);
expi1_e_fg_alt1 = SVAR_IRFS_NoFS.dsge(:,4);

IRmed_im_alt1   = SVAR_IRFS_NoFS.im;
IRmed_im0_alt1  = SVAR_IRFS_NoFS.im0;

%% ALT DGP 2: 4-period Ahead Forward Guidance Shock Allowing For Policy Reaction to Forward Guidance
x_e_fg_alt2     = SVAR_IRFS_NoFS_4period.dsge(:,1);
pi_e_fg_alt2    = SVAR_IRFS_NoFS_4period.dsge(:,2); 
i_e_fg_alt2     = SVAR_IRFS_NoFS_4period.dsge(:,3);
expi1_e_fg_alt2 = SVAR_IRFS_NoFS_4period.dsge(:,4);

IRmed_im_alt2   = SVAR_IRFS_NoFS_4period.im;
IRmed_im0_alt2  = SVAR_IRFS_NoFS_4period.im0;

figure; 
sgtitle({['Alt DGP: Allowing For Policy Reaction to Forward Guidance'],['Impulse Responses to a 1-Period Ahead Forward Guidance Shock']},'FontSize',20,'FontWeight','bold');
subplot(2,2,1);
plot(-100*x_e_fg_alt1(1:24,1),'k-','LineWidth',3.0); hold on;  plot(-100*IRmed_im_alt1(1:24,1,1),'b-.','LineWidth',3.0); plot(-100*IRmed_im0_alt1(1:24,1,1),'r--','LineWidth',3.0);
title('Output','FontSize',18);
ylabel('Percent','FontSize',16);
ylim([-0.2 1.2]); set(gca,'YTick',[-0.2:0.2:1.2],'FontSize',14);
xlim([0 24]); set(gca,'XTick',[0:4:24],'FontSize',14);

subplot(2,2,2);
plot(-400*pi_e_fg_alt1(1:24,1),'k-','LineWidth',3.0); hold on;  plot(-400*IRmed_im_alt1(1:24,2,1),'b-.','LineWidth',3.0); plot(-400*IRmed_im0_alt1(1:24,2,1),'r--','LineWidth',3.0);
title('Inflation','FontSize',18);
ylabel('Annualzied Percentage Point','FontSize',16);
ylim([-0.1 0.4]); set(gca,'YTick',[-0.1:0.1:0.4],'FontSize',14);
xlim([0 24]); set(gca,'XTick',[0:4:24],'FontSize',14);

subplot(2,2,3);
ht=plot(-400*i_e_fg_alt1(1:24,1),'k-','LineWidth',3.0); hold on;  hsfc=plot(-400*IRmed_im_alt1(1:24,3,1),'b-.','LineWidth',3.0); hs=plot(-400*IRmed_im0_alt1(1:24,3,1),'r--','LineWidth',3.0);
hleglines = [ht(1) hs(1) hsfc(1)];
leg = legend(hleglines, 'DSGE Model', 'SVAR: Pure Sign Restrictions', 'SVAR: Sign & Forecast Consistency Restrictions');
set(leg,'Location','SouthWest','FontSize',11);
title('Interest Rate','FontSize',18);
ylabel('Annualzied Percentage Point','FontSize',16);
ylim([-2.0 0.4]); set(gca,'YTick',[-2.0:0.4:0.4],'FontSize',14);
xlim([0 24]); set(gca,'XTick',[0:4:24],'FontSize',14);

subplot(2,2,4);
plot(-400*expi1_e_fg_alt1(1:24,1),'k-','LineWidth',3.0); hold on;  plot(-400*IRmed_im_alt1(1:24,4,1),'b-.','LineWidth',3.0); plot(-400*IRmed_im0_alt1(1:24,4,1),'r--','LineWidth',3.0);
title('1-Qtr Survey Forecast of Interest Rate','FontSize',18);
ylabel('Annualzied Percentage Point','FontSize',16);
ylim([-2.0 0.4]); set(gca,'YTick',[-2.0:0.4:0.4],'FontSize',14);
xlim([0 24]); set(gca,'XTick',[0:4:24],'FontSize',14);

%% PRINT FIGURE TO PDF
set(gcf, 'renderer', 'painters');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperOrientation', 'landscape');
set(gcf, 'PaperPosition', [-0.3 -0.0 12.0 8.8]);
str1 = 'SVAR_IRFS_Alt1';
str2 = '.pdf';
figname  = [str1, str2];
print([figname],'-dpdf')

clear legend;

figure; 
sgtitle({['Alt DGP: Allowing For Policy Reaction to Forward Guidance'],['Impulse Responses to a 4-Period Ahead Forward Guidance Shock']},'FontSize',20,'FontWeight','bold');
subplot(2,2,1);
plot(-100*x_e_fg_alt2(1:24,1),'k-','LineWidth',3.0); hold on;  plot(-100*IRmed_im_alt2(1:24,1,1),'b-.','LineWidth',3.0); plot(-100*IRmed_im0_alt2(1:24,1,1),'r--','LineWidth',3.0);
title('Output','FontSize',18);
ylabel('Percent','FontSize',16);
ylim([-0.2 1.2]); set(gca,'YTick',[-0.2:0.2:1.2],'FontSize',14);
xlim([0 24]); set(gca,'XTick',[0:4:24],'FontSize',14);

subplot(2,2,2);
plot(-400*pi_e_fg_alt2(1:24,1),'k-','LineWidth',3.0); hold on;  plot(-400*IRmed_im_alt2(1:24,2,1),'b-.','LineWidth',3.0); plot(-400*IRmed_im0_alt2(1:24,2,1),'r--','LineWidth',3.0);
title('Inflation','FontSize',18);
ylabel('Annualzied Percentage Point','FontSize',16);
ylim([-0.1 0.4]); set(gca,'YTick',[-0.1:0.1:0.4],'FontSize',14);
xlim([0 24]); set(gca,'XTick',[0:4:24],'FontSize',14);

subplot(2,2,3);
ht=plot(-400*i_e_fg_alt2(1:24,1),'k-','LineWidth',3.0); hold on;  hsfc=plot(-400*IRmed_im_alt2(1:24,3,1),'b-.','LineWidth',3.0); hs=plot(-400*IRmed_im0_alt2(1:24,3,1),'r--','LineWidth',3.0);
hleglines = [ht(1) hs(1) hsfc(1)];
leg = legend(hleglines, 'DSGE Model', 'SVAR: Pure Sign Restrictions', 'SVAR: Sign & Forecast Consistency Restrictions');
set(leg,'Location','SouthWest','FontSize',11);
title('Interest Rate','FontSize',18);
ylabel('Annualzied Percentage Point','FontSize',16);
ylim([-2.0 0.4]); set(gca,'YTick',[-2.0:0.4:0.4],'FontSize',14);
xlim([0 24]); set(gca,'XTick',[0:4:24],'FontSize',14);

subplot(2,2,4);
plot(-400*expi1_e_fg_alt2(1:24,1),'k-','LineWidth',3.0); hold on;  plot(-400*IRmed_im_alt2(1:24,4,1),'b-.','LineWidth',3.0); plot(-400*IRmed_im0_alt2(1:24,4,1),'r--','LineWidth',3.0);
title('4-Qtr Survey Forecast of Interest Rate','FontSize',18);
ylabel('Annualzied Percentage Point','FontSize',16);
ylim([-2.0 0.4]); set(gca,'YTick',[-2.0:0.4:0.4],'FontSize',14);
xlim([0 24]); set(gca,'XTick',[0:4:24],'FontSize',14);

%% PRINT FIGURE TO PDF
set(gcf, 'renderer', 'painters');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperOrientation', 'landscape');
set(gcf, 'PaperPosition', [-0.3 -0.0 12.0 8.8]);
str1 = 'SVAR_IRFS_Alt2';
str2 = '.pdf';
figname  = [str1, str2];
print([figname],'-dpdf')
