%% THIS SCRIPT REPLICATES FIGURE C.1 in the Appendix 
%% IN DOH AND SMITH (2021) "A NEW APPROACH TO INTEGRATING EXPECTATIONS INTO VAR MODELS"

clear all; 
close all;

%% RUN THE DSGE-VAR SIMULATIONS UNLESS THEY HAVE ALREADY BEEN SAVED
if isfile('SVAR_RMSE.mat')==0
	SVAR_MC_RMSE
else
	load 'SVAR_RMSE.mat'
end

vlags	= SVAR_RMSE(:,1);
RMSEs	= SVAR_RMSE(:,2);

figure(1);
hold on;
h1=plot(vlags,RMSEs,'b-','LineWidth',3);
title('Discrepancy Between Survey and VAR-Implied Interest Rates ','FontSize',20);
xlim([vlags(1) vlags(end)]);
set(gca,'XTick',vlags(1:1:end),'FontSize',18); 
%set(gca,'XTickLabels',{'0','0.5','1','1.5','2','2.5','3'})
%set(gca,'XTickLabels',{'0','0.5','1','1.5'})
xlabel('VAR Lag Order','FontSize',18);
ylim([0 4]);set(gca,'YTick',[0:0.5:4.0],'FontSize',18);
ylabel('RMSE','FontSize',18);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)


leg = legend([h1(1)], {'DGP: NK Model With Noisy Survey Data'});
set(leg, 'Location', 'northeast','FontSize',18);

%% PRINT FIGURE TO PDF
set(gcf, 'renderer', 'painters');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperOrientation', 'landscape');
set(gcf, 'PaperPosition', [-0.3 -0.0 12.0 8.8]);
str1 = 'RMSEFigure';
str2 = '.pdf';
figname  = [str1, str2];
print([figname],'-dpdf')