%% THIS SCRIPT REPLICATES FIGURE 7 / Figure C.2 in the Appendix
%% IN DOH AND SMITH (2021) "A NEW APPROACH TO INTEGRATING EXPECTATIONS INTO VAR MODELS"

clear all; 
close all;

%% RUN THE DSGE-VAR SIMULATIONS UNLESS THEY HAVE ALREADY BEEN SAVED
if isfile('SVAR_weights.mat')==0
	SVAR_MC_weights
else
	load 'SVAR_weights.mat'
end

%% MAKE FIGURE
plot_lambdas = SVAR_weights(:,1)./1e12;
figure;
hold on;
h1 = plot(plot_lambdas(1:end,1),SVAR_weights(1:end,2),'b-','LineWidth',3);
h2 = plot(plot_lambdas(1:end,1),SVAR_weights(1:end,3),'g:','LineWidth',3);
h3 = plot(plot_lambdas(1:end,1),SVAR_weights(1:end,4),'k-.','LineWidth',3);
h4 = plot(plot_lambdas(1:end,1),SVAR_weights(1:end,5),'r--','LineWidth',3);
leg = legend([h1(1) h2(1) h3(1) h4(1)], {'DSGE FG Shock', 'DSGE Demand Shock', ' DSGE Supply Shock', 'DSGE Noise Shock'});
set(leg, 'Location', 'southeast','FontSize',18);
title('Weight of SVAR-identified Forward Guidance Shock on Each DSGE Shock        ','FontSize',20);
xlabel('Hyperparameter Governing Tightness of Forecast Consistent Prior: \lambda (\times 10^{12})','FontSize',18);
xlim([plot_lambdas(1,1) plot_lambdas(end,1)]);
set(gca,'XTick',plot_lambdas(1:5:end,1),'FontSize',18);
ylabel('Correlation Based Weight','FontSize',18);
ylim([-1.0 1.0]);set(gca,'YTick',[-1.0:0.2:1.0],'FontSize',18);

a1 = annotation('textarrow',[0.186 0.134],[0.162 0.162],'LineWidth',2,'HeadStyle','vback2','HeadWidth',15,'HeadLength',15);
t1 = text('BackgroundColor',[1 1 1],'HorizontalAlignment','center','FontSize',16,'String',{'\lambda = 0','Sign Restrictions Only'},'Position',[0.16 -0.90 0]);

%% PRINT FIGURE TO PDF
set(gcf, 'renderer', 'painters');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperOrientation', 'landscape');
set(gcf, 'PaperPosition', [-0.3 -0.0 12.0 8.8]);
str1 = 'WeightFigure';
str2 = '.pdf';
figname  = [str1, str2];
print([figname],'-dpdf')