%% THIS SCRIPT SIMULATES DATA FROM A NEW-KEYNESIAN DSGE MODEL UNDER FIRE WITH NOISE IN THE SURVEY OBSERVATIONS
%% THEN ESTIMATES A VAR AND MEASURES THE DEGREE OF UNCONDITIONAL FORECAST CONSISTENCY

clear

% ADD VAR AUXILIARY FILES AND DYNARE TO PATH
addpath(strjoin({pwd, 'VARAuxiliaryFiles'},'\'));
addpath 'C:\dynare\4.5.7\matlab';

%% ADAPTIVE SURVEY EXPECTATIONS: 1>rho_exp>0
rho_exp_val	= 0.86; %% This is the weight on "old" information, from CG 2012 (JPE), they estimate this to be 0.86 using SPF data.

cd originalMODfiles/
save parameterfile rho_exp_val;
dynare NKModel_NoisyForecasts noclearall nolog
[result,eigenvalue_modulo,A,B,C,D] = ABCD_test(M_,options_,oo_);
fprintf(['Largest Eigenvalue of M = (A-B/D*C): ',num2str(max(eigenvalue_modulo)),'\n']);
cd ..

%% DEFINE VAR DATA
Y1			 = [x, pi, i, expi1];

%% SPECIFY LOCATION OF SURVEY FORECAST AND REALIZED DATA
s_loc	= 4;	% Location of survey measure in VAR
d_loc	= 3;	% Location of data measure in VAR

SVAR_RMSE = [];

for vv = 2:2:24;

	%% SET LAG LENGTH
	vlag = vv;

	%% ESTIMATE VAR WITH CONSTANT
	iscon = 1;

	%% INITIALIZE VAR SAMPLE
	sample_size  = 50100;
	sample_start = 100-vv;

	%% DEFINE VAR SAMPLE 
	sample_end 	 	= sample_start+sample_size;
	Y 				= Y1(sample_start:sample_end,:);

	%% ESTIMATE VAR
	[Bcomp, cvec, Bpl_ev, VC_eps, Residmat] = ...
									estim(Y, vlag, iscon);

	%% CREATE OLS MATRICES
	[Tbig,Nbig] = size(Y);
	BIG_Y 		= Y(vlag+1:Tbig,:);
	BIG_X 		= zeros(Tbig-vlag,Nbig*vlag+iscon);
	if iscon == 1
		BIG_X(:,1) 	= ones(Tbig-vlag,1);
	end;
	for lags = 1:1:vlag;
		lower_index_row		= vlag+1-lags;
		upper_index_row		= Tbig-lags;
		upper_index_col		= Nbig*lags+iscon;
		lower_index_col		= upper_index_col - Nbig + 1;
		BIG_X(:,lower_index_col:upper_index_col) = Y(lower_index_row:upper_index_row,:); 	
	end;

	%% CREATE SELECTION VECTORS;
	v_s = zeros(Nbig*vlag,1);
	v_s(s_loc) = 1;

	v_d = zeros(Nbig*vlag,1);
	v_d(d_loc) = 1;

	%% CREATE CONSTANT TERM IN 1-PERIOD AHEAD FORECAST
	if iscon == 1
		cvec_comp = [cvec; zeros(Nbig*(vlag-1),1)];
		cons_ps   = zeros(Nbig*vlag,1);
		for hh = 1:1:1;
			cons_ps = cons_ps + (Bcomp^(hh-1))*cvec_comp;
		end
	else
		cons_ps	  = zeros(Nbig*vlag,1);
	end

	%% COMPUTE RMSE OVER SAMPLE
	se = [];
	s_exp_surv = [];
	x_exp_surv = [];
	for tt = 1:1:Tbig-vlag
		X_t	 = BIG_X(tt,iscon+1:end)';
		exp_surv = v_s'*X_t;
		exp_var	 = v_d'*(cons_ps + (Bcomp^1)*X_t);
		se = [se; (exp_surv-exp_var)^2];
		s_exp_surv = [s_exp_surv; exp_surv];
		x_exp_surv = [x_exp_surv; X_t(s_loc,1)];
	end

	rmse = 100*sqrt(mean(se));
	
	SVAR_RMSE = [SVAR_RMSE; [vlag, rmse]];
	
end
	save('SVAR_RMSE','SVAR_RMSE');