%--------------------------------------------------------------------------
% Posterior VAR Simulation with Uhlig Style Sign Restrictions (2005, JME)
%--------------------------------------------------------------------------

function [allIRs, allSResids, allFactors, allSs, allSigns] = MCMCUhligDraws(Bcomp,VC_eps,cvec,R,IRhoriz,CIperc,Y,NMCDraws,NUhligDraws,NTargetAccept,A_prior,a_prior,V_prior,v_prior,S_prior);

Nbig	= length(VC_eps);
Nshocks = size(R,3);
NAccept = 0;
keep_drawingUhlig 	= 1;
keep_drawingMC  	= ~isnan(NMCDraws/NMCDraws);
mmMC				= 1;
mmUhlig 			= 1;

%% BACK OUT THE SIZE OF VARIOUS OBJECTS OF INTEREST
[Tbig,Nbig] = size(Y);
iscon  		= sum(abs(cvec))>0;
Nbig   		= length(VC_eps);
Nbigcomp 	= length(Bcomp);
vlag   		= length(Bcomp)/Nbig;

%% CREATE OLS MATRICES
BIG_Y 		= Y(vlag+1:Tbig,:);
BIG_X 		= zeros(Tbig-vlag,Nbig*vlag+iscon);
if iscon == 1
	BIG_X(:,1) 	= ones(Tbig-vlag,1);
end;
for lags = 1:1:vlag;
	lower_index_row		= vlag+1-lags;
	upper_index_row		= Tbig-lags;
	upper_index_col		= Nbig*lags+iscon;
	lower_index_col		= upper_index_col - Nbig + 1;
	BIG_X(:,lower_index_col:upper_index_col) = Y(lower_index_row:upper_index_row,:); 	
end;
%%----------------------------------------------------------------------------------------------------------------------------------
%% CALCULATE POSTERIOR DISTRIBUTION OF VAR PARAMETERS
X_temp 	    = chol(BIG_X'*BIG_X);
X_temp_inv  = inv(X_temp);
XX_temp_inv = X_temp_inv*X_temp_inv';

A_OLS 	= XX_temp_inv*(BIG_X'*BIG_Y); 
wishdof = Tbig;
scale	= Tbig - vlag - 1;			
SSE		= scale*VC_eps;
% VC_eps is calcualted using MATLAB's 
% cov() function which divides the SSE by (Tbig-vlag-1).
% Therefore, 'scale=Tbig-vlag-1' undoes this so that we 
% are working with the raw SSE

% For alpha
V_post = inv(inv(V_prior) + (BIG_X'*BIG_X));
A_post = (inv(V_prior)+(BIG_X'*BIG_X))\(inv(V_prior)*A_prior + (BIG_X'*BIG_X)*A_OLS);
a_post = reshape(A_post,Nbig*(iscon+Nbig*vlag),1);

% For SIGMA
C_S_post = chol(SSE + S_prior + A_OLS'*(BIG_X'*BIG_X)*A_OLS + A_prior'*inv(V_prior)*A_prior - A_post'*(inv(V_prior)+(BIG_X'*BIG_X))*A_post)';
S_post   = C_S_post*C_S_post';
v_post   = Tbig - vlag + v_prior;
[ans,DI] = iwishrnd(S_post,v_post); % THIS SPEEDS-UP DRAWS

%% CALCULATE POSTERIOR MEAN
Bcomp_post = [A_post(1+iscon:end,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
			  zeros((vlag-1)*Nbig, Nbig)];
SIGMA_post = S_post/(v_post-1);

%% SET INITIAL PARAMETER VALUES TO POSTERIOR MEAN
Bcomp_draw		= Bcomp_post;
RFResids_draw	= BIG_Y - BIG_X*A_post;
C_draw 			= chol(SIGMA_post)'; %% Take Cholesky Factor to Orthogonlaize Shocks

%%----------------------------------------------------------------------------------------------------------------------------------
%% DO MC & UHLIG RUNS JOINTLY (Follows Uhlig, 2005 JME, Pg. 411)
while keep_drawingMC == 1 || keep_drawingUhlig == 1;
	
	%% DO MCMC DRAW
	while keep_drawingMC == 1
		%% HERE I USE ANTITHETIC SAMPLING
		if mod(mmMC,2) == 1
			% odd draw again
			SIGMA_draw 		= iwishrnd(S_post,v_post,DI);
			vec_BIG_A_COVAR	= kron(SIGMA_draw,V_post);
			vec_A_draw_u	= mvnrnd(zeros(Nbig*(1+Nbig*vlag),1)',vec_BIG_A_COVAR);
			a_draw			= a_post + vec_A_draw_u'; 
			
		else
			% even draw - take reflection
			a_draw	= a_post - vec_A_draw_u'; 
		end;
		
		%% PUT IN COMPANION FORM
		BIG_A_draw		= reshape(a_draw,iscon+Nbig*vlag,Nbig);
		Bcomp_draw		=[BIG_A_draw(1+iscon:end,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
									zeros((vlag-1)*Nbig, Nbig)];
		RFResids_draw	= BIG_Y - BIG_X*BIG_A_draw;
		C_draw   		= chol(SIGMA_draw)'; %% Take Cholesky Factor to Orthogonlaize Shocks
		
		mmMC = mmMC + 1;
		keep_drawingMC = 0;
	end;
	
	%% DO UHLIG DRAW FOR GIVEN REDUCED FORM VAR PARAMETERS
	while keep_drawingUhlig == 1;
		%workbar(max(NAccept,1)/NTargetAccept,'Performing Uhlig Draws w/ MCMC','Progress');
	
		%% ROTATE CHOLESKY FACTOR RANDOMLY (CALL THIS ROTATION 'FACTOR')
		% HERE WE FOLLOW KILIAN AND LUTKEPOHL (Pg. 427) INSTEAD OF UHLIG TO GENERALIZE TO MULTIPLE SHOCKS
		%[Q,R] 	= qr(reshape(mvnrnd(zeros(Nbig*Nbig,1)',eye(Nbig*Nbig)),Nbig,Nbig));
		%S		= Q*diag(sign(diag(R)));
		S = OrthNorm(Nbig);
		Factor  = C_draw*S';
		
		%% CALCULATE IRFS
		IR = VAR_irf_factor(Bcomp_draw, Factor, IRhoriz);
		
		%% DECIDE TO KEEP OR REJECT ROTATION BASED ON SIGN RESTRICTIONS
		checkall = ones(Nbig,Nshocks);
		checkall_flip = ones(Nbig,Nshocks);
		for ss=1:Nshocks
			for ii = 1:Nbig
				if R(ii,1,ss) ~= 0
					if R(ii,3,ss) == 1
						check = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) > 0;
						checkall(ii,ss) = min(check);
						% Check flipped signs
						check_flip = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) < 0;
						checkall_flip(ii,ss) = min(check_flip);
					elseif R(ii,3,ss) == -1
						check = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) < 0;
						checkall(ii,ss) = min(check);
						% Check flipped signs
						check_flip = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) > 0;
						checkall_flip(ii,ss) = min(check_flip);
					end
				end
			end
		end

		%% IF RESTRICTIONS ARE SATISFIED, STORE IRFS
		if min(min(checkall))==1
			NAccept = NAccept + 1;
			allIRs(:,:,:,NAccept) = IR;
			inv_Factor = inv(Factor);
			allSResids(:,:,NAccept) = RFResids_draw*inv_Factor';
			allFactors(:,:,NAccept) = Factor;
			allSs(:,:,NAccept)		= S;
			allSigns(:,NAccept)		= 1;
			%workbar(NAccept/NTargetAccept,'Performing Uhlig Draws w/ MCMC','Progress');
		elseif min(min(checkall_flip))==1
			NAccept = NAccept + 1;
			allIRs(:,:,:,NAccept) = -1*IR;
			inv_Factor = inv(Factor);
			allSResids(:,:,NAccept) = -1*RFResids_draw*inv_Factor';
			allFactors(:,:,NAccept) = -1*Factor;
			allSs(:,:,NAccept)		= -1*S;
			allSigns(:,NAccept)		= -1;
			%workbar(NAccept/NTargetAccept,'Performing Uhlig Draws w/ MCMC','Progress');
		end
		
		%% CHECK WHETHER TO EXIT UHLIG DRAWING LOOP OR CONTINUE IN UHLIG DRAWING LOOP
		if NAccept == NTargetAccept
			keep_drawingMC 	  = 0;
			keep_drawingUhlig = 0;
			disp('***** Targeted Number of Accepted Draws Reached *****');
			disp(['Percentage of Joint Draws Accepted:	' num2str(100*NAccept/(mmMC*mmUhlig))]);
		elseif NAccept < NTargetAccept & mmUhlig >= NUhligDraws
			keep_drawingUhlig = 0;
		elseif NAccept < NTargetAccept & mmUhlig < NUhligDraws
			mmUhlig = mmUhlig + 1;
		end
	end
	
	%% CHECK WHETHER TO EXIT JOINT DRAWING LOOP OR CONTINUE IN JOINT DRAWING LOOP
	if NAccept < NTargetAccept & mmMC >= NMCDraws
		keep_drawingMC 	  = 0;
		%workbar(1,'Performing Uhlig Draws w/ MCMC','Progress');	
		disp('***** Max Number of Draws Reached *****');
		disp(['Progress Towards Targeted Number of Draws:	' num2str(100*NAccept/NTargetAccept)]);
		disp(['Percentage of Joint Draws Accepted:			' num2str(100*NAccept/(NMCDraws*NUhligDraws))]);	
	elseif NAccept < NTargetAccept & mmMC < NMCDraws
		keep_drawingMC 	  = 1;
		keep_drawingUhlig = 1;
		mmUhlig = 1;
	end
	
end;