

function [IR] = VAR_irf_factor(Bcomp, Factor, IRhoriz);


%--------------------------------------------------------------------------
%
% DESCRIPTION:
%
% This routine computes and plots the impulse response function
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Bcomp:   matrix with structure of estimated reduced-form coefficients in 
%           companion form excluding all the deterministic terms. 
%           This includes the estimated parameters in the partition 
%           n x (n x (n x vlag)). The remaining partition is made up of a 
%           diagonalic identity matrix that pins down the lead-lag relation 
%           between variables. The size is (n x vlag) x (n x vlag).  
%
% Factor:  Matrix of contemporaneous structural relationships between variables
%		    For Cholesky --> Factor = chol(VC_eps)'; 
%
% IRhoriz: number of periods for which point impulse responses are computed
%
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% IR:      impulse responses (IRhoriz x n x n) 
%
%--------------------------------------------------------------------------



Nbig       = length(Factor);
Nbigcomp   = length(Bcomp);

%VC_epschol  = chol(VC_eps)'; 
%shockvecmat = VC_epschol;

shockvecmat = Factor; 

[nz,nofIR]         = size(shockvecmat);
aux                = zeros(Nbigcomp, Nbigcomp);
aux(1:Nbig,1:Nbig) = shockvecmat;
shockvecmat        = aux;

IR     = zeros(IRhoriz+1, Nbig, nofIR);
Impmat = eye(length(Bcomp));


for hh=1:IRhoriz+1;
    IRbig  = Impmat*shockvecmat;
    IR(hh, 1:Nbig, 1:Nbig) = IRbig(1:Nbig, 1:Nbig);
    Impmat = Impmat*Bcomp;
end

