function [Q,flag] = get_4d_orthonorm(a1,a2,a3,a4)

%% Normalize first column
U0 = [a1;a2;a3;a4];
norm(U0);
U0 = U0./norm(U0);
[~,sort_id] = sort(abs(U0),1,'descend');
%U0 = U0(sort_id);

x0 = U0(1,1);
y0 = U0(2,1);
z0 = U0(3,1);
w0 = U0(4,1);

%% Construct second column
U1 = [y0;-x0;0;0];
if norm(U1) < 1e-12
	U1 = [0;z0;-y0;0];
end;
U1 = U1./norm(U1);

x1 = U1(1,1);
y1 = U1(2,1);
z1 = U1(3,1);
w1 = U1(4,1);

%% Construct third column
%% First define 6 determinant cases
det_cases = [
x0*y1 - x1*y0;
x0*z1 - x1*z0;
x0*w1 - x1*w0;
y0*z1 - y1*z0;
y0*w1 - y1*w0;
z0*w1 - z1*w0];

[~,loc] = max(abs(det_cases));

Id = eye(4);

E0 = Id(:,1);
E1 = Id(:,2);
E2 = Id(:,3);
E3 = Id(:,4);

%% Six cases to consider
%% First assume loc == 1, then we need 0's in x,y location
if loc == 1;
	mat2 = [
	U0';
	U1';
	E2']; 
elseif loc == 2;
	mat2 = [
	U0';
	U1';
	E1']; 
elseif loc == 3;
	mat2 = [
	U0';
	U1';
	E1']; 
elseif loc == 4;
	mat2 = [
	U0';
	U1';
	E0']; 
elseif loc == 5;
	mat2 = [
	U0';
	U1';
	E0']; 
elseif loc == 6
	mat2 = [
	U0';
	U1';
	E0']; 
end

submat20 = mat2(:, logical([0,1,1,1]));
submat21 = mat2(:, logical([1,0,1,1]));
submat22 = mat2(:, logical([1,1,0,1]));
submat23 = mat2(:, logical([1,1,1,0]));

U2 = det(submat20)*E0 - det(submat21)*E1 + det(submat22)*E2 - det(submat23)*E3;

norm(U2);

U2 = U2./norm(U2);

x2 = U2(1,1);
y2 = U2(2,1);
z2 = U2(3,1);
w2 = U2(4,1);

%% Construct fourth column
mat3 = [
	U0';
	U1';
	U2']; 

submat30 = mat3(:, logical([0,1,1,1]));
submat31 = mat3(:, logical([1,0,1,1]));
submat32 = mat3(:, logical([1,1,0,1]));
submat33 = mat3(:, logical([1,1,1,0]));

U3 = det(submat30)*E0 - det(submat31)*E1 + det(submat32)*E2 - det(submat33)*E3;

norm(U3);

U3 = U3./norm(U3);

x3 = U2(1,1);
y3 = U2(2,1);
z3 = U2(3,1);
w3 = U2(4,1);

Q = [U0, U1, U2, U3];

if max(abs(reshape((eye(4) - Q'*Q),16,1))) > 1e-12
	flag = 1;
else
	flag = 0;
end;