function [R2] = get_shock_R2(Y, vlag, iscon, DSGE_shock);

                                
%--------------------------------------------------------------------------
%
% DESCRIPTION:
%
% This routine produces the regression r-squared of a DSGE (DGP) shock of interest
% in order to assess the degree of invertibility
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Y:     matrix of data series, m x n
%
% vlag:  lag length   
%
% iscon: a value '1' introduces a constant in the conditional mean 
%
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% R2:	regression r-squared from regressing DSGE shock of interest on vlags of macro aggregates
%
%--------------------------------------------------------------------------
%
% Author:  A. Lee Smith, December 2021
%
%--------------------------------------------------------------------------


%-------------------------
cvec    = 0;

[Tbig0,Nbig] = size(Y);
Tbig         = Tbig0-vlag;
%-------------------------


%--------------------------------------------------------------------------
allY  = zeros(Tbig, (vlag+1)*Nbig);

uplcc = 1; 

for ii=0:vlag;
    allY(:,uplcc:uplcc+Nbig-1) = Y(1+vlag-ii:Tbig0-ii, 1:Nbig);
    uplcc = uplcc+Nbig;
end

allYlag = allY(:, Nbig+1:end);
Y       = allY(:, 1:Nbig);

Xmat    = allYlag;

if iscon==1;
    Xmat = [Xmat ones(Tbig, 1)];
    cvec = zeros(Nbig, 1);
end
%--------------------------------------------------------------------------


%--------------------------------------------------------------------------

[~, ~, ~, ~, stats_loc] = regress(DSGE_shock(vlag+1:end,1), Xmat);

R2 = stats_loc(1,1);






