%% THIS FUNCTION PLOTS THE BEST AND WORST DRAWS

function plotIRF2model(allIRs,CIperc,s_loc,d_loc,mm_best,mm_worst,save_string)
clear legend;

%% DEFINE PERIOD TO PLOT
IRhoriz = size(allIRs,1);	% Length of IRF
irflength = IRhoriz-13;		% Length of IRF to plot
t = 1:irflength;

%% GET LOWER, MEDIAN AND UPPER VARIABLES OF INTEREST FROM SIMULATED DISTRIBUTION
%% IRFS
for hh=1:size(allIRs,1);
    for ii=1:size(allIRs,2);
        for jj=1:size(allIRs,3);
            sliceloc   =  squeeze(allIRs(hh,ii,jj,:));
            IRpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            IRlo(hh,ii,jj)   = IRpercsloc(1);
			IRmed(hh,ii,jj)  = IRpercsloc(2);
            IRup(hh,ii,jj)   = IRpercsloc(3);
        end
    end
end

IRbest  = allIRs(:,:,:,mm_best);
IRworst = allIRs(:,:,:,mm_worst);

var1y_best  = NaN(irflength,1);
var1y_worst = NaN(irflength,1);
for hh = 13:1:IRhoriz-1;
	var1y_best(hh-12)  = mean(IRbest(hh-1:hh+1,d_loc,1));
	var1y_worst(hh-12) = mean(IRworst(hh-1:hh+1,d_loc,1));
end;

IRdev_best  = var1y_best  - IRbest(1:size(var1y_best),s_loc,1);
IRdev_worst = var1y_worst - IRworst(1:size(var1y_worst),s_loc,1);

figure
subplot(3,2,1)
hold on
plot(t,-IRbest(1:irflength,1,1),'-.g','LineWidth',3);
plot(t,-IRworst(1:irflength,1,1),'--r','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Output','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.1 0.4]);set(gca,'YTick',[-0.1:0.1:0.4],'FontSize',12);
ylabel('Percent','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(3,2,2)
hold on
hbw1=plot(t,-cumsum(IRdev_best),'-.g','LineWidth',3);
hbw2=plot(t,-cumsum(IRdev_worst),'--r','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Cumulative Forecast Deviation','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.1 0.4]);set(gca,'YTick',[-0.1:0.1:0.4],'FontSize',12);
ylabel('APR','FontSize',12);
hleglines = [hbw1(1) hbw2(1)];
leg = legend(hleglines, '"Best" Fit','"Worst" Fit');
set(leg,'Location','NorthWest','FontSize',8); 
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(3,2,3)
hold on
plot(t,-IRbest(1:irflength,d_loc,1),'-.g','LineWidth',3);
plot(t,-IRworst(1:irflength,d_loc,1),'--r','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Fed Funds','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.1 0.4]);set(gca,'YTick',[-0.1:0.1:0.4],'FontSize',12);
ylabel('APR','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

subplot(3,2,4)
hold on
plot(t,-IRbest(1:irflength,s_loc,1),'-.g','LineWidth',3);
plot(t,-IRworst(1:irflength,s_loc,1),'--r','LineWidth',3);
%plot(t,-var1y,'--r','LineWidth',3);
plot(t,zeros(irflength,1), '-k', 'LineWidth', 1); 
title('Blue Chip 4-Qtr Ahead Rate','FontSize',16);xlim([t(1) t(end)]);set(gca,'XTick',t(12:12:end),'FontSize',12);
%ylim([-0.1 0.05]);set(gca,'YTick',[-0.1:0.05:0.05],'FontSize',12);
ylabel('APR','FontSize',12);
box on; grid on;
set(gca, 'Layer', 'top','GridLineStyle',':','GridColor',[.4, .4, .4],'GridAlpha',1)

set(gcf, 'renderer', 'painters');
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperOrientation', 'portrait');
set(gcf, 'PaperPosition', [-0.3 -0.7 9.5 12.0]);
print([save_string,'.pdf'],'-dpdf')

