function [SA,notes,eps] = GIBBS1(S0,P0,P1,T,N,L);

% function SA = GIBBS1(Y,X,S0,P0,P1,T,N,L);

% Edited on March 21, 2018 by Lee Smith
% replaced sqrtm with chol to speed-up calculation

% This file performs the first step of the Gibbs sampler,
% drawing from p(theta | Q,R,Y) 

% initialize arrays for Gibbs sampler
SA = zeros(N*(1+N*L),T); % artificial states
SM = zeros(N*(1+N*L),1); % backward update for conditional mean of state vector
PM = zeros(N*(1+N*L),N*(1+N*L)); % backward update for projection matrix
P = PM; % backward update for covariance matrix
wa = randn(N*(1+N*L),T); % draws for state innovations
notes=zeros(1,T);
eps=zeros(1,T); 

% Check the non-positve definite matrix 
[CLT,critp] = chol(P0(:,:,T),'lower');

if critp ~= 0
	notes(T) = critp;
	CL0 = zeros(N*(1+N*L),N*(1+N*L))
	CL0(1:critp-1,1:critp-1) = CLT;
	CLT = CL0;
end; 

% Backward recursions and sampling
% Terminal state
SA(:,T) = S0(:,T) + CLT*wa(:,T);

% iterating back through the rest of the sample
for i = 1:T-1,
   PM = P0(:,:,T-i)*inv(P1(:,:,T-i+1));
   P = P0(:,:,T-i) - PM*P0(:,:,T-i);
   SM = S0(:,T-i) + PM*(SA(:,T-i+1) - S0(:,T-i));
   
   [CL,critp] = chol(P,'lower');
	if critp ~= 0
		notes(i) = critp;
		CL0 = zeros(N*(1+N*L),N*(1+N*L))
		CL0(1:critp-1,1:critp-1) = CL;
		CL = CL0;
	end; 
   
   SA(:,T-i) = SM + CL*wa(:,T-i);
   
end
   
% Updating SI,PI
%PM = P0(:,:,1)*inv(P1(:,:,1));
%PI = P0(:,:,1) - PM*P0(:,:,1);
%SI = S0(:,1) + PM*(SA(:,1) - S0(:,1));
   
   