function h=numHess(fcn,x,delta,bnd,varargin)
% translated from numHess2.R by WYP, 6/14/2011
%
% delta can be a scalar or a vector of increments. A scalar delta is applied 
% to all the parameters.
% When bnd==1 and the function returns +- infinity, a function value in the
% opposite direction (if not infinity) is used instead.
% This works best when fcn is reparameterized so that it is unrestricted.
% This does not handle cases where fcn returns multiple values.
% needs a column vector for x.
% adjusted step size the last term: June 30, 2016 Taeyoung Doh 
nx = length(x);
h = zeros(nx,nx);
f0 = feval(fcn,x,varargin{:});
nd = length(delta);
% build an increment matrix
if (nd==1)
    delta = delta*ones(nx,1);
    if nx >11
    delta(1:4)=1e-2; 
    end 
    if nx>15 
    delta(5:6) = 1e-2;
    delta(7:8) = 1;  
    end 
        
    delta(nx)=1e-8; 
    tvec = diag(delta); 
else
    if (nx==nd)
        tvec = diag(delta);
    else
        error('Check the length of delta')
    end
end
% diagonals first
for indi=1:nx
    tvecv = tvec(:,indi);
    f1 = feval(fcn,x+tvecv,varargin{:});
    f2 = feval(fcn,x-tvecv,varargin{:});
    fprintf('[%2i] [f+] %12.6f %1i [f-] %12.6f %1i [f0] %12.6f\n',indi,f1,(f1>f0),f2,(f2>f0),f0);
    if (bnd)
        if (isinf(f1) && ~isinf(f2))
            disp('... [f+] is replaced by [f-]')
            f1 = f2;
        elseif (~isinf(f1) && isinf(f2))
            disp('... [f-] is replaced by [f+]')
            f2 = f1;
        elseif (isinf(f1) && isinf(f2))
            disp('... [f+] and [f-] are INFs. No good.')
        end
    end
    h(indi,indi) = (f1+f2-2*f0)/delta(indi)^2;
end
% off-diagonals next
if (nx > 1)
    for indi=1:(nx-1)
        for indj=(indi+1):nx
            tvecvi = tvec(:,indi);
            tvecvj = tvec(:,indj);
            tvecvij = tvec(:,indi) - tvec(:,indj);
            f11 = feval(fcn,x+tvecvi,varargin{:});
            f12 = feval(fcn,x-tvecvi,varargin{:});
                if (bnd)
                    if (isinf(f11) && ~isinf(f12))
                        disp('... [f1+] is replaced by [f1-]')
                        f11 = f12;
                    elseif (~isinf(f11) && isinf(f12))
                        disp('... [f1-] is replaced by [f1+]')
                        f12 = f11;
                    elseif (isinf(f11) && isinf(f12))
                        disp('... [f1+] and [f1-] are INFs. No good.')
                    end
                end
            f21 = feval(fcn,x+tvecvj,varargin{:});
            f22 = feval(fcn,x-tvecvj,varargin{:});
                if (bnd)
                    if (isinf(f21) && ~isinf(f22))
                        disp('... [f2+] is replaced by [f2-]')
                        f21 = f22;
                    elseif (~isinf(f21) && isinf(f22))
                        disp('... [f2-] is replaced by [f2+]')
                        f22 = f21;
                    elseif (isinf(f21) && isinf(f22))
                        disp('... [f2+] and [f2-] are INFs. No good.')
                    end
                end  
            f31 = feval(fcn,x+tvecvij,varargin{:});
            f32 = feval(fcn,x-tvecvij,varargin{:});
                if (bnd)
                    if (isinf(f31) && ~isinf(f32))
                        disp('... [f1+2-] is replaced by [f1-2+]')
                        f31 = f32;
                    elseif (~isinf(f31) && isinf(f32))
                        disp('... [f1-2+] is replaced by [f1+2-]')
                        f32 = f31;
                    elseif (isinf(f31) && isinf(f32))
                        disp('... [f1+2-] and [f1-2+] are INFs. No good.')
                    end 
                end   
            h(indi,indj) = .5*(f11 + f12 + f21 + f22 - f31 - f32 - 2*f0)/delta(indi)/delta(indj);
            h(indj,indi) = h(indi,indj);
            % below is just to show the shape of the function
            f1 = feval(fcn,x+tvecvi+tvecvj,varargin{:});
            f2 = feval(fcn,x-tvecvi-tvecvj,varargin{:});
            fprintf('[%2i][%2i] [f++] %12.6f %1i [f--] %12.6f %1i [f0] %12.6f\n',indi,indj,f1,(f1>f0),f2,(f2>f0),f0);
        end
    end
end