function h=numHess2(fcn,x,delta,bnd,verbose,varargin)
% translated from numHess2.R by WYP, 6/14/2011
%
% delta can be a scalar or a vector of increments. A scalar delta is applied 
% to all the parameters.
% When bnd==1 and the function returns +- infinity, a function value in the
% opposite direction (if not infinity) is used instead.
% This works best when fcn is reparameterized so that it is unrestricted.
% This does not handle cases where fcn returns multiple values.
% needs a column vector for x.
%
nx = length(x);
nd = length(delta);
% build an increment matrix
if (nd==1)
    delta = delta*ones(nx,1);
    tvec = diag(delta);
else
    if (nx==nd)
        tvec = diag(delta);
        tveci = find(delta~=0);
        nd = length(tveci);
        nx = nd;
    else
        error('Check the length of delta')
    end
end
h = zeros(nx,nx);
f0 = feval(fcn,x,varargin{:});
% diagonals first
for indi=1:nx
    tvecv = tvec(:,tveci(indi));
    f1 = feval(fcn,x+tvecv,varargin{:});
    f2 = feval(fcn,x-tvecv,varargin{:});
    if (verbose), fprintf('[%2i] [f+] %12.6f %1i [f-] %12.6f %1i [f0] %12.6f\n',indi,f1,(f1>f0),f2,(f2>f0),f0); end;
    if (bnd)
        if (isinf(f1) && ~isinf(f2))
            if (verbose), disp('... [f+] is replaced by [f-]'); end;
            f1 = f2;
        elseif (~isinf(f1) && isinf(f2))
            if (verbose), disp('... [f-] is replaced by [f+]'); end;
            f2 = f1;
        elseif (isinf(f1) && isinf(f2))
            if (verbose), disp('... [f+] and [f-] are INFs. No good.'); end;
        end
    end
    h(indi,indi) = (f1+f2-2*f0)/delta(tveci(indi))^2;
end
% off-diagonals next
if (nx > 1)
    for indi=1:(nx-1)
        for indj=(indi+1):nx
            tvecvi = tvec(:,tveci(indi));
            tvecvj = tvec(:,tveci(indj));
            tvecvij = tvec(:,tveci(indi)) - tvec(:,tveci(indj));
            f11 = feval(fcn,x+tvecvi,varargin{:});
            f12 = feval(fcn,x-tvecvi,varargin{:});
                if (bnd)
                    if (isinf(f11) && ~isinf(f12))
                        if (verbose), disp('... [f1+] is replaced by [f1-]'); end;
                        f11 = f12;
                    elseif (~isinf(f11) && isinf(f12))
                        if (verbose), disp('... [f1-] is replaced by [f1+]'); end;
                        f12 = f11;
                    elseif (isinf(f11) && isinf(f12))
                        if (verbose), disp('... [f1+] and [f1-] are INFs. No good.'); end;
                    end
                end
            f21 = feval(fcn,x+tvecvj,varargin{:});
            f22 = feval(fcn,x-tvecvj,varargin{:});
                if (bnd)
                    if (isinf(f21) && ~isinf(f22))
                        if (verbose), disp('... [f2+] is replaced by [f2-]'); end;
                        f21 = f22;
                    elseif (~isinf(f21) && isinf(f22))
                        if (verbose), disp('... [f2-] is replaced by [f2+]'); end;
                        f22 = f21;
                    elseif (isinf(f21) && isinf(f22))
                        if (verbose), disp('... [f2+] and [f2-] are INFs. No good.'); end;
                    end
                end  
            f31 = feval(fcn,x+tvecvij,varargin{:});
            f32 = feval(fcn,x-tvecvij,varargin{:});
                if (bnd)
                    if (isinf(f31) && ~isinf(f32))
                        if (verbose), disp('... [f1+2-] is replaced by [f1-2+]'); end;
                        f31 = f32;
                    elseif (~isinf(f31) && isinf(f32))
                        if (verbose), disp('... [f1-2+] is replaced by [f1+2-]'); end;
                        f32 = f31;
                    elseif (isinf(f31) && isinf(f32))
                        if (verbose), disp('... [f1+2-] and [f1-2+] are INFs. No good.'); end;
                    end 
                end   
            h(indi,indj) = .5*(f11 + f12 + f21 + f22 - f31 - f32 - 2*f0)/delta(tveci(indi))/delta(tveci(indj));
            h(indj,indi) = h(indi,indj);
            % below is just to show the shape of the function
            f1 = feval(fcn,x+tvecvi+tvecvj,varargin{:});
            f2 = feval(fcn,x-tvecvi-tvecvj,varargin{:});
            if (verbose), fprintf('[%2i][%2i] [f++] %12.6f %1i [f--] %12.6f %1i [f0] %12.6f\n',indi,indj,f1,(f1>f0),f2,(f2>f0),f0); end;
        end
    end
end