% fn_inv_nrstr_const.m
% Function to compute the quarterly growth rate tracking estimates of
% the construction component of nonresidential structures investment.

function [f_g_inv_nrstr_ex_mining,f_g_inv_nrstr_ex_mining_next_q]=fn_inv_nrstr_const(date_q, ...
	g_inv_nrstr_ex_mining,inv_nrstr_ex_mining_n,inv_nrstr_ex_mining, ...
	date_m,f_const_nonres,f_ppi3,ff);
	
global junk_nos;

% compute imputed price index
inv_nrstr_ex_mining_imp_p=junk_nos*ones(size(date_q));
i1=inv_nrstr_ex_mining~=junk_nos & inv_nrstr_ex_mining_n~=junk_nos;
inv_nrstr_ex_mining_imp_p(i1)=inv_nrstr_ex_mining_n(i1)./inv_nrstr_ex_mining(i1);
% compute 4-quarter change in imputed price index
g4_inv_nrstr_ex_mining_imp_p=junk_nos*ones(size(date_q));
for i1=min(find(inv_nrstr_ex_mining_imp_p~=junk_nos))+4:max(find(inv_nrstr_ex_mining_imp_p~=junk_nos));
	g4_inv_nrstr_ex_mining_imp_p(i1)=100*(inv_nrstr_ex_mining_imp_p(i1)/inv_nrstr_ex_mining_imp_p(i1-4)-1);
end;
% compute 1-step ahead imputed price index and gross growth rates for 
% the 1-step ahead measure
f1_inv_nrstr_ex_mining_imp_p=junk_nos*ones(size(date_q));
gg_f1_inv_nrstr_ex_mining_imp_p=junk_nos*ones(size(date_q));
for i1=min(find(g4_inv_nrstr_ex_mining_imp_p~=junk_nos))+1:length(date_q);
	if inv_nrstr_ex_mining_imp_p(i1-1)~=junk_nos & g4_inv_nrstr_ex_mining_imp_p(i1-1)~=junk_nos;
		f1_inv_nrstr_ex_mining_imp_p(i1)=inv_nrstr_ex_mining_imp_p(i1-1)*((1+g4_inv_nrstr_ex_mining_imp_p(i1-1)/100)^.25);
		if f1_inv_nrstr_ex_mining_imp_p(i1-1)~=junk_nos & f1_inv_nrstr_ex_mining_imp_p(i1)~=junk_nos;
			gg_f1_inv_nrstr_ex_mining_imp_p(i1)=f1_inv_nrstr_ex_mining_imp_p(i1)/f1_inv_nrstr_ex_mining_imp_p(i1-1);
		end;
	end;
end;
% compute quarterly averages for nonresidential construction put in place and ppis
f_const_nonres_q=fn_convert_mtoqavg(f_const_nonres,date_m,date_q);
for i1=1:3;
	f_ppi_q(:,i1)=fn_convert_mtoqavg(f_ppi3(:,i1),date_m,date_q);
end;
% compute gross growth rates for ppis
gg_f_ppi_q=junk_nos*ones(size(date_q,1),size(f_ppi_q,2));
for i1=1:3;
	for i2=min(find(f_ppi_q(:,i1)~=junk_nos))+1:max(find(f_ppi_q(:,i1)~=junk_nos));
		gg_f_ppi_q(i2,i1)=f_ppi_q(i2,i1)/f_ppi_q(i2-1,i1);
	end;
end;
% compute means across ppis
mean_gg_ppi_q=junk_nos*ones(size(date_q));
for i1=1:length(date_q);
	temp=gg_f_ppi_q(i1,:);
	temp=temp(temp~=junk_nos);
	if isempty(temp)==0;
		mean_gg_ppi_q(i1)=mean(temp);
	end;
end;
% take average of gross growth rates for ppi measure and 1-step ahead measure
mean_gg_p2=junk_nos*ones(size(date_q));
i1=gg_f1_inv_nrstr_ex_mining_imp_p~=junk_nos & mean_gg_ppi_q==junk_nos;
mean_gg_p2(i1)=gg_f1_inv_nrstr_ex_mining_imp_p(i1);
i1=gg_f1_inv_nrstr_ex_mining_imp_p~=junk_nos & mean_gg_ppi_q~=junk_nos;
mean_gg_p2(i1)=0.5*(gg_f1_inv_nrstr_ex_mining_imp_p(i1)+mean_gg_ppi_q(i1));
i1=gg_f1_inv_nrstr_ex_mining_imp_p==junk_nos & mean_gg_ppi_q~=junk_nos;
mean_gg_p2(i1)=mean_gg_ppi_q(i1);
% roll forward the imputed price level 1 period
f1_inv_nrstr_ex_mining_imp_p_new=junk_nos*ones(size(date_q));
for i1=min(find(inv_nrstr_ex_mining_imp_p~=junk_nos))+1:length(date_q);
	if mean_gg_p2(i1)~=junk_nos & inv_nrstr_ex_mining_imp_p(i1-1)~=junk_nos;
		f1_inv_nrstr_ex_mining_imp_p_new(i1)=inv_nrstr_ex_mining_imp_p(i1-1)*mean_gg_p2(i1);
	end;
	if mean_gg_p2(i1)~=junk_nos & inv_nrstr_ex_mining_imp_p(i1-1)==junk_nos & f1_inv_nrstr_ex_mining_imp_p_new(i1-1)~=junk_nos;
		f1_inv_nrstr_ex_mining_imp_p_new(i1)=f1_inv_nrstr_ex_mining_imp_p_new(i1-1)*mean_gg_p2(i1);
	end;
end;
% deflate nonresidential construction put in place by the imputed price deflator
f_const_nonres_q_r=junk_nos*ones(size(date_q));
i1=inv_nrstr_ex_mining_imp_p~=junk_nos & f_const_nonres_q~=junk_nos;
f_const_nonres_q_r(i1)=f_const_nonres_q(i1)./inv_nrstr_ex_mining_imp_p(i1);
% deflate nonresidential construction put in place by the constructed price deflator
ff_const_nonres_q_r=junk_nos*ones(size(date_q));
i1=f_const_nonres_q~=junk_nos & f1_inv_nrstr_ex_mining_imp_p_new~=junk_nos;
ff_const_nonres_q_r(i1)=f_const_nonres_q(i1)./f1_inv_nrstr_ex_mining_imp_p_new(i1);
% compute quarterly annualized growth rates 
f_g_inv_nrstr_ex_mining=junk_nos*ones(size(date_q));
for i1=min(find(f_const_nonres_q_r~=junk_nos))+1:length(date_q);
	f_g_inv_nrstr_ex_mining(i1)=100*((ff_const_nonres_q_r(i1)/f_const_nonres_q_r(i1-1))^4-1);
end;
% check
if length(date_q)==length(g_inv_nrstr_ex_mining);
zout=[date_q,g_inv_nrstr_ex_mining,f_g_inv_nrstr_ex_mining];
end;
% forecast growth for the next quarter
f_const_nonres_next_q=mean(f_const_nonres(ff+3:ff+5));
mean_gg_p2_next_q=mean(mean(f_ppi3(ff+3:ff+5,:))./mean(f_ppi3(ff:ff+2,:)));
f1_inv_nrstr_ex_mining_imp_p_new_next_q=f1_inv_nrstr_ex_mining_imp_p_new(length(date_q))*mean_gg_p2_next_q;
ff_const_nonres_next_q_r=f_const_nonres_next_q/f1_inv_nrstr_ex_mining_imp_p_new_next_q;
f_g_inv_nrstr_ex_mining_next_q=100*((ff_const_nonres_next_q_r/ff_const_nonres_q_r(length(date_q)))^4-1);