function [S0,P0,P1] = KFRQvar(Y,X,B,Q,C,H,SI,PI,T,N,L);

% function [S0,P0,P1] = KFRQvar(Y,X,B,Q,C,H,SI,PI,T,N,L);

% This file performs the forward kalman filter recursions
% for the random coefficients VAR.  R,Q are time varying, depending
% on the stochastic volatilities.
% RR = inv(C)*H*inv(C) is univariate
% QQ = inv(B)*Q(t)*inv(B)

% SI,PI are the initial values for the recursions, S(1|0) and P(1|0)

S0 = zeros(N*(1+N*L),T); % current estimate of the state, S(t|t)
P0 = zeros(N*(1+N*L),N*(1+N*L),T); % current estimate of the covariance matrix, P(t|t)
P1 = zeros(N*(1+N*L),N*(1+N*L),T); % one-step ahead covariance matrix, P(t|t-1)

BI = inv(B);
CI = inv(C);

% date 1
RR = CI*diag(H(2,:))*CI';
QQ = BI*diag(Q(2,:))*BI'; % shifted one period rel to observables
P1(:,:,1) = PI + QQ; % P(1|0)
K = (P1(:,:,1)*X(:,:,1))*inv( X(:,:,1)'*P1(:,:,1)*X(:,:,1) + RR); % K(1)
P0(:,:,1) = P1(:,:,1) - K*(X(:,:,1)'*P1(:,:,1)); % P(1|1)
S0(:,1) = SI + K*( Y(:,1) - X(:,:,1)'*SI ); % S(1|1)

% Iterating through the rest of the sample
for i = 2:T,
   RR = CI*diag(H(i+1,:))*CI'; % shifted one period rel to observables
   QQ = BI*diag(Q(i+1,:))*BI'; % shifted one period rel to observables
   P1(:,:,i) = P0(:,:,i-1) + QQ; % P(t|t-1)
   K = (P1(:,:,i)*X(:,:,i))*inv( X(:,:,i)'*P1(:,:,i)*X(:,:,i) + RR); % K(t)
   P0(:,:,i) = P1(:,:,i) - K*(X(:,:,i)'*P1(:,:,i)); % P(t|t)
   S0(:,i) = S0(:,i-1) + K*( Y(:,i) - X(:,:,i)'*S0(:,i-1) ); % S(t|t)
end
