%  This file generates marginal likelihood 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VAR, Core PCE Inflation

%clear


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 
DFILE = 'posterior.mat'; 
%save(DFILE,'SD','QD','HD','CD','VD','LD','SS','INDEX_RW','SWT','SA','SQ','SH','SC','SV','SL','SSS'); 

%load(DFILE,'SD')    % time-varying parameters 
load(DFILE,'SA')    % time-varying parameters without resampling 
%load(DFILE,'QD')    % covariance matrix for innovations to TVPs 
load(DFILE,'SQ')    % covariance matrix for innovations to TVPs without resampling 
%load(DFILE,'HD')    % stochastic volatility 
load(DFILE,'SH')    % stochastic volatility without resampling 
%load(DFILE,'VD')    % standard deviations of innovations to log volatility  
load(DFILE,'SV')    % stadnadr deviations of innovations to log volatility without resampling  
%load(DFILE,'CD')    % loadings for measurement innovation variance  
load(DFILE,'SC')    % loadings for measurement innovation variance without resampling 
%load(DFILE,'LD')    % Cholesky factorization 
load(DFILE,'SL')    % Cholesky factorization without resampling 
%load(DFILE,'SS')    % Strucutral shocks 
load(DFILE,'SSS')   % Structural shocks without resampling 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


randn('state',100)

[NK,TP,P] = size(SA);
[NQ,NQ,P]= size(SQ);
[HT,NH,P]=size(SH);
[NV,P]=size(SV);
[NC,P]=size(SC);

%  FIRST LOAD THE DATA

ts		= importdata('data_with_labels.xlsx');    
ts_data	= ts.data;
dates	= 1970.25:0.25:2017.75;
dates 	= dates';

%% DEFINE VARIABLES
p10e	= ts_data(:,1); % 400*LN(1+CPI LONG-TERM FORWARD EXP/400)	 
p1e	= ts_data(:,2);	% 400*LN(1+CPI 1 YEAR AHEAD EXP/400)
p	 	= ts_data(:,3); % 400*LN(1+CPI QUARTERLY PRECENT CHANGE AT AN ANNUALZIED RATE /400)
act  	= ts_data(:,4); % CHICAGO FED NATIONAL ACTIVITY INDEX (QUARTERLY AVERAGE OF MONTHLY VALUES)
int  	= ts_data(:,5); % 400*LN(1+FED FUNDS RATE/400) NOTE: FROM 2009:Q1-2015:Q4 WU-XIA SHADOW RATE IS USED

%% ORDER VARIABLES IN VAR (FOLLOWS FROM CLARK AND DAVIG'S IDENTIFICATION)
y 		= [p10e, p1e, p, act, int]; % 
y         = y/400;
 
L = 4; % VAR lag order 
[T,N] = size(y);   % T: Total Sample Size (1970:Q2-2017:Q4), N: Number of variables (5) 
% create the quarterly indicator to distinguish different horizon length of long-term forward inflation forecasts 
% 35 (qindex=1), 34 (qindex=2) , 33 (qindex=3), 32 (qindex=4)  
qindex = 1+(dates-floor(dates))./0.25;

% adjust for lags, create rh variables
% lagged dependent variables: X1 = [y(t-1) y(t-1) ... y(t-L)]
X1 = zeros(T-L,1+(N*L));
X1(1:T-L,1) = ones(T-L,1);

for i = 1:L 
	X1(1:T-L,2+(N*(i-1)):1+(i*N)) = y(1+L-i:T-i,:);
end

Y = y(1+L:T,:)'; % Third row to the end, then transpose  : N by T-L
X = XMAT_5(X1',X1',X1',X1',X1');   % 
% partitioning the data
% initial estimates through T0
Train = 49+L; % 1970:Q2 - 1982:Q4 
T0 = Train-L; %adjust for training sample
Y0 = Y(:,1:T0);  
YS = Y(:,1+T0:T-L);
Y_aux = Y(:,T0-2:T-L)';
X0 = X(:,:,1:T0);
XS = X(:,:,1+T0:T-L);
X01 = X1(1:T0,:);
XS1 = X1(1+T0:T-L,:);


% Prior distribution for Q, Sig_e , B 
[SI,PI,RI] = SUR(Y0,X0,T0); 

% priors for sv (inverse gamma) (standard dev for volatility innovation)
sv0 = .002;
v0 = 1;
d0 = sv0^2;

% priors for log h0 and correlation parameters (normal, ballpark numbers)
ss0 = 10;                             
mu0 = log(diag(RI));

% prior for correlation parameters for stochastic volatilities (normal)
ch0 = zeros(N*(N-1)/2,1);
ch = ch0;
ssc0 = 10000*eye(N*(N-1)/2,N*(N-1)/2);

% prior for Q 
df = N*(1+N*L) + 1; % prior degrees of freedom  : controls the tightness of priors for the time-variation in coefficients 
DF = df + T - L - T0; % posterior degrees of freedom 
% (cogley-sargent) : control implied variation in trend
% implied prior mean and AR roots
S = [eye(N*(L-1)),zeros(N*(L-1),N)];
ASI = reshape(SI,1+N*L,N);
A = [ASI(2:end,:)'; S];
rr = max(abs(eig(A)));
A0 = [ASI(1,:)' ; zeros(N*(L-1),1)];
M = inv(eye(N*L) - A)*A0;

% Calibration of the shrinkage parameter for Q0 : (added by Taeyoung Doh) 
SAA = SI + real(sqrtm(PI))*ones(N*(1+N*L),1);   % One stadard deviation shocks for all the coefficients 
AA = [SAA(2:1+N*L,1)'; SAA(2+(1+N*L):2*(1+N*L),1)'; SAA(2+2*(1+N*L):3*(1+N*L),1)';SAA(2+3*(1+N*L):4*(1+N*L))';SAA(2+4*(1+N*L):5*(1+N*L))';S];
AA0 =  [SAA(1,1); SAA(2+N*L,1); SAA(3+2*N*L,1);SAA(4+3*N*L,1);SAA(5+4*N*L,1); zeros(N*(L-1),1)]; 
MA = inv(eye(N*L) - AA)*AA0;
dMA = MA-M; 
adj = min(diag(cov(Y0'))./(T0*(dMA(1:N,1).^2)));
Q0 = adj*PI;   % The variance of the change in the trend during T0 period is less than or equal to the variances of observed variables during the pre-sample period. 
% Calibration over! 
TQ0 = df*Q0; 
TQ  = TQ0; 


WW      =  var_weight(SI,PI,N,L);    
LFF     =  zeros(TP,P); 
%lgrid = 0.05:0.05:1.5; 
%tl = length(lgrid); 

%txx=1; 
%mml = zeros(txx,1); 
tic;
%for k=1:txx
lambda = 1.5;  
%for tt=1:T
for jj=1:P 
    LFF(:,jj) = im_weight(SA(:,:,jj),N,L,WW,qindex(end-137:end),lambda); 
end
% Select draws that have non-negligible probabilities

%index = find(mean(LFF)>-76.084);  % 
%lindex = length(index); 
%if lindex>0.1*length(LFF)
LX =exp(mean(LFF))./sum(exp(mean(LFF)));
neff = 1/sum(LX.^2); 
disp('effective sample size: lambda neff')
[lambda neff]
 if neff>0.02*P    % effective sample size>100
INDEX_RW = randsample(1:P,P,true,LX'); 
 SD = SA(:,:,INDEX_RW); 
 QD = SQ(:,:,INDEX_RW); 
 HD = SH(:,:,INDEX_RW);
 CD = SC(:,INDEX_RW);
 VD = SV(:,INDEX_RW);
 LD = SL(:,:,INDEX_RW);
 SS = SSS(:,:,INDEX_RW);
else 
disp('too few draws to resample: decrease lambda'); 
return; 
end 
toc;
mlik = zeros(P,1);  
for iter=1:P 

evar = INNOVM(YS,XS1,SD(:,:,iter),N,T-T0-L,L);
    lik = zeros(T-T0-L,1); 
    for jj=1:T-T0-L 
    lik(jj) =  -0.5*N*log(2*pi)-0.5*log(abs(det(inv(LD(:,:,iter))*diag(HD(jj+1,:,iter))*inv(LD(:,:,iter))')))-0.5*evar(:,jj)'*inv(inv(LD(:,:,iter))*diag(HD(jj+1,:,iter))*inv(LD(:,:,iter))')*evar(:,jj);
    end       
    mlik(iter)=sum(lik);
end
[mlik,index]= sort(mlik); 
mlik = mlik(0.16*P:0.84*P); 
densfac = mlik(1)+700; 
emlik = exp(-mlik+densfac);

% Calculate marginal prior probability of the truncated region 
 lp =zeros(length(emlik),1); 
 for mj=0.16*P:0.84*P
     kk=index(mj); 
% Step 1: theta 
M=100;
QM = zeros(N*(N*L+1),N*(N*L+1),M); 
mlp1 = zeros(M,1); 

for jj=1:M 
   QM(:,:,jj) = gibbs2Q(TQ0,df,N,L);  
   QM(:,:,jj) = 0.5*(QM(:,:,jj)+QM(:,:,jj)'); 
   RR = cholcov(QM(:,:,jj));
   PI = 0.5*(PI+PI'); 
   for t=1:T-T0-L
       if t==1 
       lp1 = -0.5*length(SI)*log(2*pi)-sum(log(diag(RR)))-0.5*(SD(:,t,kk)-SI)'*inv(QM(:,:,jj))*(SD(:,t,kk)-SI);    
       else    
       lp1 = lp1 -0.5*length(SI)*log(2*pi)-sum(log(diag(RR)))-0.5*(SD(:,t,kk)-SD(:,t-1,kk))'*inv(QM(:,:,jj))*(SD(:,t,kk)-SD(:,t-1,kk));  
       end
   end
    mlp1(jj)=lp1;   
end
den1 = max(mlp1)-700;
ind = find(mlp1>den1); 
mlp1 = mlp1(ind); 
lp1 = log(sum(exp(mlp1-den1)/M))+den1;
% Step 2: sigma 
lp2 = -0.5*length(diag(RI))*log(2*pi)-0.5*log(abs(det(10*eye(N))))-0.5*(log(HD(1,:,kk))'-log(diag(RI)))'*inv(10*eye(N))*(log(HD(1,:,kk))'-log(diag(RI)))+sum(log(gampdf(1./(VD(:,kk).^2),v0,1/d0)));
   for t=1:T-T0-L 
        lp2 = lp2 -0.5*N*log(2*pi)-0.5*log(abs(det(diag(VD(:,kk).^2))))-0.5*(log(HD(t+1,:,kk))'-log(HD(t,:,kk))')'*inv(diag(VD(:,kk).^2))*(log(HD(t+1,:,kk))'-log(HD(t,:,kk))'); 
   end     

% Step 3: B 
LL = squeeze([LD(2,1,kk) LD(3,1,kk) LD(3,2,kk) LD(4,1,kk) LD(4,2,kk) LD(4,3,kk) LD(5,1,kk) LD(5,2,kk) LD(5,3,kk) LD(5,4,kk)])'; 
lp3 = -0.5*length(ch0)*log(2*pi)-0.5*log(abs(det(ssc0)))-0.5*LL'*inv(ssc0)*LL;
lp(mj) = lp1+lp2+lp3; 
end 
 
dens=max(lp)-700;
mml = -log(mean(emlik))+densfac-log(mean(exp(lp-dens)))-dens; 
%end 

% [B,I]=max(mml); 
% 
% mlambda = lgrid(I);   
% for jj=1:P 
%     LFF(:,jj) = im_weight(SA(:,:,jj),N,L,WW,qindex(end-137:end),lambda); 
% end
% 
% 
% LX =exp(mean(LFF(:,index)))./sum(exp(mean(LFF(:,index))));
% INDEX_RW = randsample(1:P,P,true,LX'); 
%  SD = SA(:,:,INDEX_RW); 
%  QD = SQ(:,:,INDEX_RW); 
%  HD = SH(:,:,INDEX_RW);
%  CD = SC(:,INDEX_RW);
%  VD = SV(:,INDEX_RW);
%  LD = SL(:,:,INDEX_RW);
%  SS = SSS(:,:,INDEX_RW);
% 
% 
% 
 sfile =['TVP_VAR_ml_lambda_alt2.mat']; 
 save(sfile); 



