%  This file generates long-run (10 year ahead) forecasts


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VAR, Core PCE Inflation

clear


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 
DFILE = 'posterior_alt.mat'; 
%save(DFILE,'SD','QD','HD','CD','VD','LD','SS','INDEX_RW','SWT','SA','SQ','SH','SC','SV','SL','SSS'); 

load(DFILE,'SD')    % time-varying parameters 
load(DFILE,'SA')    % time-varying parameters without resampling 
load(DFILE,'QD')    % covariance matrix for innovations to TVPs 
load(DFILE,'SQ')    % covariance matrix for innovations to TVPs without resampling 
load(DFILE,'HD')    % stochastic volatility 
load(DFILE,'SH')    % stochastic volatility without resampling 
load(DFILE,'VD')    % standard deviations of innovations to log volatility  
load(DFILE,'SV')    % stadnadr deviations of innovations to log volatility without resampling  
load(DFILE,'CD')    % loadings for measurement innovation variance  
load(DFILE,'SC')    % loadings for measurement innovation variance without resampling 
load(DFILE,'LD')    % Cholesky factorization 
load(DFILE,'SL')    % Cholesky factorization without resampling 
load(DFILE,'SS')    % Strucutral shocks 
load(DFILE,'SSS')   % Structural shocks without resampling 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

randn('state',100)

[NK,T,P] = size(SD);
[NQ,NQ,P]= size(QD);
[HT,NH,P]=size(HD);
[NV,P]=size(VD);
[NC,P]=size(CD);

%  FIRST LOAD THE DATA
rawdata = xlsread('Datasource_revised.xlsx','P262:AD499');  
infdata = xlsread('raw_tsp.xlsx','B2:DO239'); 
int = rawdata(:,1);   % shadow rate 
pi2 = rawdata(:,4);   % 2 year inf. exp. 
pi10 = rawdata(:,end-1); % 5yr5yr forward 
inf = rawdata(:,end-2);  % headline inflation 
act = rawdata(:,end-4);  % Chicago Fed activitiy index 
ffr = rawdata(:,end);    % federal funds rate 
subs = 131; 
zlb=84; 
inta = ffr;           
inta(subs+1:subs+zlb)=int(subs+1:subs+zlb);  % replace ffr by shadow rate only for the ZLB period 
mat = 3:1:120; 


% Quarterly VAR 
pi10q = zeros(floor(length(pi10)/3),1); 
pi2q  = zeros(floor(length(pi2)/3),1); 
infq  = zeros(floor(length(inf)/3),1); 
actq  = zeros(floor(length(act)/3),1); 
intq  = zeros(floor(length(int)/3),1); 
intqa = zeros(floor(length(inta)/3),1); 
% quarterly averaing 
for kk=1:floor(length(pi10)/3) 
  pi10q(kk) = mean(pi10(3*kk-2:3*kk)); 
  pi2q(kk)  = mean(pi2(3*kk-2:3*kk)); 
  infq(kk)  = mean(inf(3*kk-2:3*kk)); 
  actq(kk)  = mean(act(3*kk-2:3*kk));
  intq(kk)  = mean(int(3*kk-2:3*kk)); 
  intqa(kk) = mean(inta(3*kk-2:3*kk)); 
end 

datasel = 1; % 1 for Doh and Choi (2016) shadow rate, 2 for Doh and Choi (2016) shadow rate only for the ZLB period, 3 for Wu and Xia shadow rate only for the ZLB period 

if datasel ==1  
yy = [pi10q pi2q infq actq intq]; 
elseif datasel ==2 
yy = [pi10q pi2q infq actq intqa]; 
elseif datasel ==3 
yy = [pi10q pi2q infq actq intqwx]; 
end 
% date
dates = 1998:1/12:2017+9/12;
dates = dates'; 

%% DEFINE VARIABLES
pi10e	= pi10q;
pi1e	= pi2q;
pi	 	= infq;
act  	= actq;
int  	= yy(:,5); 

%% ORDER VARIABLES IN VAR (FOLLOWS FROM CLARK AND DAVIG'S IDENTIFICATION)
y 		= [pi10e, pi1e, pi, act, int]; % 
y         = y/400;


 
L = 1; % VAR lag order 
[T,N] = size(y);   % T: Total Sample Size (1970:Q2-2017:Q4), N: Number of variables (5) 


% adjust for lags, create rh variables
% lagged dependent variables: X1 = [y(t-1) y(t-1) ... y(t-L)]
X1 = zeros(T-L,1+(N*L));
X1(1:T-L,1) = ones(T-L,1);

for i = 1:L 
	X1(1:T-L,2+(N*(i-1)):1+(i*N)) = y(1+L-i:T-i,:);
end

Y = y(1+L:T,:)'; % Third row to the end, then transpose  : N by T-L
X = XMAT_5(X1',X1',X1',X1',X1');   % 
% partitioning the data
% initial estimates through T0
Train = 20+L; % 1970:Q2 - 1982:Q4 
T0 = Train-L; %adjust for training sample
Y0 = Y(:,1:T0);  
YS = Y(:,1+T0:T-L);
Y_aux = Y(:,T0-2:T-L)';
X0 = X(:,:,1:T0);
XS = X(:,:,1+T0:T-L);
X01 = X1(1:T0,:);
XS1 = X1(1+T0:T-L,:);
% sample runs from 2003Q3 to 2017Q3

data = YS; % ordered as in benchmark model for D&V
date = 2003.5:1/4:2017+3/4;
[ND1,ND2]=size(data');
randn('state',100)


%FH        = 5;  % forecast horizon 
%TH        = ND1; % predictive forecast 
%SAF= zeros(NK,FH,(NF-NB)*P);

%SHF= zeros(FH,NH,(NF-NB)*P);
%YYF= zeros(ND2,FH,(NF-NB)*P); 


[NK,T,NMC] = size(SA);


% Trend Consumption

mm  = zeros(N,T);
mmr = zeros(N,T); 
tpi10 = zeros(T,NMC);
tpi10r = zeros(T,NMC); 
mtpi10 =zeros(NMC,1);
mtpi10r = zeros(NMC,1); 
tpi2 = zeros(T,NMC); 
tpi2r = zeros(T,NMC); 
mtpi2 = zeros(NMC,1); 
mtpi2r = zeros(NMC,1); 
tpi = zeros(T,NMC); 
tpir =zeros(T,NMC); 
mtpi = zeros(NMC,1); 
mtpir =zeros(NMC,1); 
tact = zeros(T,NMC); 
tactr = zeros(T,NMC); 
mtact = zeros(NMC,1);
mtactr = zeros(NMC,1); 
tint  = zeros(T,NMC);
tintr = zeros(T,NMC); 
mtint = zeros(NMC,1); 
mtintr = zeros(NMC,1); 
Sigmat = zeros(NV,NV,NMC); 
Sigmatr = zeros(NV,NV,NMC); 
vol     = zeros(NV,T,NMC); 
volr    = zeros(NV,T,NMC); 
pred    = zeros(NV,T,NMC); 
predr   = zeros(NV,T,NMC);
spi10 = [1 0 0 0 0]; spi2 = [0 1 0 0 0]; spi =[0 0 1 0 0]; sact =[0 0 0 1 0]; sint =[0 0 0 0 1]; 

for i = 1:NMC,
    for t = 1:T,
       A = [SA(2:1+N*L,t,i)'; SA(2+(1+N*L):2*(1+N*L),t,i)'; SA(2+2*(1+N*L):3*(1+N*L),t,i)'; SA(2+3*(1+N*L):4*(1+N*L),t,i)';SA(2+4*(1+N*L):5*(1+N*L),t,i)'];
       Ar = [SD(2:1+N*L,t,i)'; SD(2+(1+N*L):2*(1+N*L),t,i)'; SD(2+2*(1+N*L):3*(1+N*L),t,i)'; SD(2+3*(1+N*L):4*(1+N*L),t,i)';SD(2+4*(1+N*L):5*(1+N*L),t,i)'];
       M = inv(eye(N*L) - A)*[SA(1,t,i); SA(2+N*L,t,i); SA(3+2*N*L,t,i);SA(4+3*N*L,t,i);SA(5+4*N*L,t,i)];
       Mr = inv(eye(N*L) - Ar)*[SD(1,t,i); SD(2+N*L,t,i); SD(3+2*N*L,t,i);SD(4+3*N*L,t,i);SD(5+4*N*L,t,i)];
       mm(:,t) = M(1:N,:); 
       mmr(:,t) = Mr(1:N,:);
       Sigmat(:,:,i) = inv(SL(:,:,i))'*diag((SH(t,:,i)))*inv(SL(:,:,i)); 
       Sigmatr(:,:,i)= inv(LD(:,:,i))'*diag((HD(t,:,i)))*inv(LD(:,:,i));
       vol(:,t,i) = sqrt(diag(Sigmat(:,:,i))); 
       volr(:,t,i) = sqrt(diag(Sigmatr(:,:,i))); 
       sigmay = doubling(Sigmat(:,:,i),A,eps); 
       sigmayr = doubling(Sigmatr(:,:,i),Ar,eps); 
       pred(:,t,i) = diag(A*sigmay*A')./diag(sigmay); 
       predr(:,t,i) = diag(Ar*sigmayr*Ar')./diag(sigmayr); 
    end
   tpi10(:,i) = (spi10*mm)';     tpi10r(:,i) = (spi10*mmr)'; 
   tpi2(:,i)  = (spi2*mm)';      tpi2r(:,i)  = (spi2*mmr)';  
   tpi(:,i)   = (spi*mm)';       tpir(:,i)   = (spi*mmr)'; 
   tact(:,i)  = (sact*mm)';      tactr(:,i) = (sact*mmr)';  
   tint(:,i)  = (sint*mm)';      tintr(:,i) = (sint*mmr)'; 
end

sspi10  = sort(tpi10,2);
sspi10r = sort(tpi10r,2); 
sspi2   = sort(tpi2,2);
sspi2r  = sort(tpi2r,2); 
sspi    = sort(tpi,2); 
sspir   = sort(tpir,2);
ssact   = sort(tact,2); 
ssactr  = sort(tactr,2); 
ssint   = sort(tint,2); 
ssintr  = sort(tintr,2); 


IQR1 = [sspi10(:,.16*NMC),sspi10(:,.5*NMC),sspi10(:,.84*NMC)];   
IQR1r = [sspi10r(:,.16*NMC),sspi10r(:,.5*NMC),sspi10r(:,.84*NMC)]; 
IQR2 = [sspi2(:,.16*NMC),sspi2(:,.5*NMC),sspi2(:,.84*NMC)];
IQR2r = [sspi2r(:,.16*NMC),sspi2r(:,.5*NMC),sspi2r(:,.84*NMC)]; 
IQR3 = [sspi(:,.16*NMC),sspi(:,.5*NMC),sspi(:,.84*NMC)];
IQR3r = [sspir(:,.16*NMC), sspir(:,.5*NMC), sspir(:,.84*NMC)]; 
IQR4  = [ssact(:,.16*NMC),ssact(:,.5*NMC),ssact(:,.84*NMC)]; 
IQR4r = [ssactr(:,.16*NMC),ssactr(:,.5*NMC),ssactr(:,.84*NMC)]; 
IQR5  = [ssint(:,.16*NMC), ssint(:,.5*NMC), ssint(:,.84*NMC)];
IQR5r = [ssintr(:,.16*NMC), ssintr(:,.5*NMC), ssintr(:,.84*NMC)]; 

figure;
subplot(3,3,1)
%plot(date(:),400*(IQR1(:,1)),'--k','LineWidth',2); hold on;
plot(date(:),400*mean(tpi10,2),'-k','LineWidth',2); hold on; 
plot(date(:),400*mean(tpi10r,2),'-r','LineWidth',2); hold on; 
%plot(date(:),400*(IQR1(:,3)),'--k','LineWidth',2); hold on;
title('LR Inf: mean','Fontsize',14)

subplot(3,3,2)
%plot(date(:),400*(IQR1r(:,1)),'--r','LineWidth',2); hold on;
plot(date(:),400*mean(tpi2,2),'-k','LineWidth',2); hold on; 
plot(date(:),400*mean(tpi2r,2),'-r','LineWidth',2); hold on; 
%plot(date(:),400*(IQR1r(:,3)),'--r','LineWidth',2); hold on;
title('SR Inf: mean','Fontsize',14)

subplot(3,3,3)
%plot(date(:),400*(IQR2(:,1)),'--k','LineWidth',2); hold on;
plot(date(:),400*mean(tpi,2),'-k','LineWidth',2); hold on;
plot(date(:),400*mean(tpir,2),'-r','LineWidth',2); hold on; 
%plot(date(:),400*(IQR2(:,3)),'--k','LineWidth',2); hold on;
title('Inf: mean','Fontsize',14)

subplot(3,3,4)
%plot(date(:),400*(IQR2r(:,1)),'--r','LineWidth',2); hold on;
plot(date(:),400*mean(vol(1,:,:),3)','-k','LineWidth',2); hold on; 
plot(date(:),400*mean(volr(1,:,:),3)','-r','LineWidth',2); hold on; 
%plot(date(:),400*(IQR2r(:,3)),'--r','LineWidth',2); hold on;
title('LR Inf: vol','Fontsize',14)

subplot(3,3,5)
%plot(date(:),400*(IQR2r(:,1)),'--r','LineWidth',2); hold on;
plot(date(:),400*mean(vol(2,:,:),3)','-k','LineWidth',2); hold on; 
plot(date(:),400*mean(volr(2,:,:),3)','-r','LineWidth',2); hold on; 
%plot(date(:),400*(IQR2r(:,3)),'--r','LineWidth',2); hold on;
title('SR Inf: vol','Fontsize',14)

subplot(3,3,6)
%plot(date(:),400*(IQR2r(:,1)),'--r','LineWidth',2); hold on;
plot(date(:),400*mean(vol(3,:,:),3)','-k','LineWidth',2); hold on; 
plot(date(:),400*mean(volr(3,:,:),3)','-r','LineWidth',2); hold on; 
%plot(date(:),400*(IQR2r(:,3)),'--r','LineWidth',2); hold on;
title('Inf: vol','Fontsize',14)

subplot(3,3,7)
%plot(date(:),400*(IQR2r(:,1)),'--r','LineWidth',2); hold on;
plot(date(:),mean(pred(1,:,:),3)','-k','LineWidth',2); hold on; 
plot(date(:),mean(predr(1,:,:),3)','-r','LineWidth',2); hold on; 
%plot(date(:),400*(IQR2r(:,3)),'--r','LineWidth',2); hold on;
title('LR Inf: predictability','Fontsize',14)

subplot(3,3,8)
%plot(date(:),400*(IQR2r(:,1)),'--r','LineWidth',2); hold on;
plot(date(:),mean(pred(2,:,:),3)','-k','LineWidth',2); hold on; 
plot(date(:),mean(predr(2,:,:),3)','-r','LineWidth',2); hold on; 
%plot(date(:),400*(IQR2r(:,3)),'--r','LineWidth',2); hold on;
title('SR Inf: predictability','Fontsize',14)

subplot(3,3,9)
%plot(date(:),400*(IQR2r(:,1)),'--r','LineWidth',2); hold on;
plot(date(:),mean(pred(3,:,:),3)','-k','LineWidth',2); hold on; 
plot(date(:),mean(predr(3,:,:),3)','-r','LineWidth',2); hold on; 
%plot(date(:),400*(IQR2r(:,3)),'--r','LineWidth',2); hold on;
title('Inf: predictability','Fontsize',14)



print(figure,'-dpdf','trendinf.pdf')


% 
% FH         = 40;  % forecast horizon  
% TH         = length(date);  % the recent 50 years 
% SAF= zeros(NK,FH,P);     % without resampling 
% SAFA = zeros(NK,FH,P);   % wiht resampling 
% SHF= zeros(FH,NH,P);     % without resampling    
% SHFA = zeros(FH,NH,P);   % with resampling  
% YYF= zeros(N,FH,P);    % without resampling  
% YYFA=zeros(N,FH,P);    % with resampling 
% YYY = zeros(N,FH,TH,P); %  without resampling 
% YYYA = zeros(N,FH,TH,P); % with resampling 
% Y25  = zeros(N,TH);  
% Y50  = zeros(N,TH); 
% Y75  = zeros(N,TH); 
% Y25A = zeros(N,TH); 
% Y50A = zeros(N,TH); 
% Y75A = zeros(N,TH); 
% T=TH;
% for jj=1:TH
%   tic;
%     
% for i = 1:P   % parameter uncertainty 
%    
%     
%     % Step 1: Innovations to time-varying coefficients 
%       
%     for tt=1:FH
%         if tt==1   
%      SAF(:,tt,i) = SA(:,T-TH+jj,i)+chol(SQ(:,:,i),'lower')*randn(NQ,1); 
%      SAFA(:,tt,i) = SD(:,T-TH+jj,i)+chol(QD(:,:,i),'lower')*randn(NQ,1); 
%     %  SAF(:,tt,i)=SA(:,T-TH+jj,i);
%     %  SAFA(:,tt,i)=SD(:,T-TH+jj,i); 
%         else 
%      SAF(:,tt,i) = SAF(:,tt-1,i)+chol(SQ(:,:,i),'lower')*randn(NQ,1);
%      SAFA(:,tt,i) =SAFA(:,tt-1,i)+chol(QD(:,:,i),'lower')*randn(NQ,1);   
%     %     SAF(:,tt,i)=SAF(:,tt-1,i);
%     %  SAFA(:,tt,i)=SAFA(:,tt-1,i); 
%        
%         end 
%      end         
%   
%     
%     % Step 2: Innovations to stochastic volatility 
%     for tt=1:FH
%         if tt==1  
%     SHF(tt,:,i) = log(SH(T-TH+jj,:,i))+(diag(SV(:,i))*randn(NV,1))';   
%     SHF(tt,:,i) = exp(SHF(tt,:,i)); 
%     %SHF(tt,:,i) = SH(T-TH+jj,:,i); 
%     SHFA(tt,:,i) = log(HD(T-TH+jj,:,i))+(diag(VD(:,i))*randn(NV,1))';
%     SHFA(tt,:,i) = exp(SHFA(tt,:,i)); 
%     %SHFA(tt,:,i) = HD(T-TH+jj,:,i);  
%         else 
%     SHF(tt,:,i) = log(SHF(tt-1,:,i))+(diag(SV(:,i))*randn(NV,1))';
%     SHF(tt,:,i) = exp(SHF(tt,:,i));
%     SHFA(tt,:,i) = log(SHFA(tt-1,:,i))+(diag(VD(:,i))*randn(NV,1))'; 
%     SHFA(tt,:,i) = exp(SHFA(tt,:,i));  
%         end 
%     end 
%     
%     % Step 3: Innovations to equation errors 
%     for tt=1:FH 
%         innov=inv(SL(:,:,i))*diag(sqrt(SHF(tt,:,i)))*randn(N,1);
%         innov2=inv(LD(:,:,i))*diag(sqrt(SHFA(tt,:,i)))*randn(N,1);  
%       
%        theta = reshape(SAF(:,tt,i),1+N*L,N)';
%       theta_a = reshape(SAFA(:,tt,i),1+N*L,N)';
%         if tt==1  
%        YYF(:,tt,i) = theta(:,1)+theta(:,2:end)*data(:,jj)+innov;
%     YYFA(:,tt,i)= theta_a(:,1)+theta_a(:,2:end)*data(:,jj)+innov2;      
%         else 
%        YYF(:,tt,i) = theta(:,1)+theta(:,2:end)*YYF(:,tt-1,i)+innov;
%     YYFA(:,tt,i)= theta_a(:,1)+theta_a(:,2:end)*YYFA(:,tt-1,i)+innov2;                   
%         end 
%     end   
%     YYY(:,:,jj,i) = YYF(:,:,i); 
%     YYYA(:,:,jj,i) = YYFA(:,:,i); 
% end
%  for kk=1:N
%  Y50(kk,jj) = quantile(mean(YYY(kk,21:40,jj,:),2),0.5);                                            % 10 year ahead forecast 
%  Y25(kk,jj) = quantile(mean(YYY(kk,21:40,jj,:),2),0.25);
%  Y75(kk,jj) = quantile(mean(YYY(kk,21:40,jj,:),2),0.75);
%  Y50A(kk,jj)= quantile(mean(YYYA(kk,21:40,jj,:),2),0.5); 
%  Y25A(kk,jj)= quantile(mean(YYYA(kk,21:40,jj,:),2),0.25); 
%  Y75A(kk,jj)= quantile(mean(YYYA(kk,21:40,jj,:),2),0.75); 
%  end
%  toc; 
% end
% sfile =['trend_forecast_altdata.mat']; 
% 
% 
% save(sfile,'Y50','Y75','Y25','YYY','YYYA','Y25A','Y50A','Y75A'); 
% 
% 
% for kk=1:5
%   if kk==4
%      kk=kk+1;
%   end 
% figure;
% subplot(2,1,1); 
%     fill([date fliplr(date)], 400*[Y75(kk,:) fliplr(Y25(kk,:))],[0.5 0.5 0.5]); 
%     hold on; 
% plot(date,400*Y50(kk,:),'k',date,400*Y25(kk,:),'b--',date,400*Y75(kk,:),'b--','LineWidth',2);    
% grid on; 
% hold off; 
%  ylabel('% percent')
% legend('IQR','Median')
% xlim([date(1) date(end)])
% subplot(2,1,2); 
%  fill([date fliplr(date)], 400*[Y75A(kk,:) fliplr(Y25A(kk,:))],[0.5 0.5 0.5]); 
%     hold on; 
% plot(date,400*Y50A(kk,:),'k',date,400*Y25A(kk,:),'b--',date,400*Y75A(kk,:),'b--','LineWidth',2);    
% grid on; 
% hold off; 
%   ylabel('% percent')
% legend('IQR: with resampling','Median: with resampling')
% xlim([date(1) date(end)])
% if kk==1 title('Long-run Inflation Expectations')
% elseif kk==2 title ('Short-run Inflation Expectations')
% elseif kk==3 title ('CPI Inflation')
% elseif kk==5 title ('Interest Rate')
%  end
% end 
% 
% % Term Structrue of Inflation Tail Risk 
% 
% pL  = squeeze(mean(YYY(1,1:20,:,:),2));
% pS  = squeeze(mean(YYY(2,1:20,:,:),2)); 
% p1  = squeeze(mean(YYY(3,1:4,:,:),2));
% p2  = squeeze(mean(YYY(3,1:8,:,:),2));
% p3  = squeeze(mean(YYY(3,1:12,:,:),2));
% p5   = squeeze(mean(YYY(3,1:20,:,:),2));
% p7  = squeeze(mean(YYY(3,1:28,:,:),2));
% p10 = squeeze(mean(YYY(3,1:40,:,:),2));
% pLa  = squeeze(mean(YYYA(1,1:20,:,:),2));
% pSa  = squeeze(mean(YYYA(2,1:20,:,:),2)); 
% 
% pa1  = squeeze(mean(YYYA(3,1:4,:,:),2));
% pa2  = squeeze(mean(YYYA(3,1:8,:,:),2));
% pa3  = squeeze(mean(YYYA(3,1:12,:,:),2));
% pa5   = squeeze(mean(YYYA(3,1:20,:,:),2));
% pa7  = squeeze(mean(YYYA(3,1:28,:,:),2));
% pa10 = squeeze(mean(YYYA(3,1:40,:,:),2));
% p = p5;
% pa =pa5;
% infts =zeros(T,6,2);
% inftsa = zeros(T,6,2);
% 
% infL  = zeros(T,4); 
% infS  = zeros(T,4);  
% inf   = zeros(T,4);
% 
% infLa = zeros(T,4); 
% infSa = zeros(T,4); 
% infa  = zeros(T,4); 
% for j=1:T
% infL(j,1) = mean(400*pL(j,:)'>4); infL(j,2) = mean(400*pL(j,:)'>3); infL(j,3) = mean(400*pL(j,:)'<1); infL(j,4) = mean(400*pL(j,:)'<0); 
% infS(j,1) = mean(400*pS(j,:)'>4); infS(j,2) = mean(400*pS(j,:)'>3); infS(j,3) = mean(400*pS(j,:)'<1); infS(j,4) = mean(400*pS(j,:)'<0); 
% inf(j,1)  = mean(400*p(j,:)'>4);  inf(j,2)  = mean(400*p(j,:)'>3);  inf(j,3)  = mean(400*p(j,:)'<1);  inf(j,4)  = mean(400*p(j,:)'<0); 
% infLa(j,1) = mean(400*pLa(j,:)'>4); infLa(j,2) = mean(400*pLa(j,:)'>3); infLa(j,3) = mean(400*pLa(j,:)'<1); infLa(j,4) = mean(400*pLa(j,:)'<0); 
% infSa(j,1) = mean(400*pSa(j,:)'>4); infSa(j,2) = mean(400*pSa(j,:)'>3); infSa(j,3) = mean(400*pSa(j,:)'<1); infSa(j,4) = mean(400*pSa(j,:)'<0); 
% infa(j,1)  = mean(400*pa(j,:)'>4);  infa(j,2)  = mean(400*pa(j,:)'>3);  infa(j,3)  = mean(400*pa(j,:)'<1);  infa(j,4)  = mean(400*pa(j,:)'<0); 
% infts(j,1,1) = mean(400*p1(j,:)'>4); infts(j,1,2)=mean(400*p1(j,:)'<0);
% infts(j,2,1) = mean(400*p2(j,:)'>4); infts(j,2,2)=mean(400*p2(j,:)'<0);
% infts(j,3,1) = mean(400*p3(j,:)'>4); infts(j,3,2)=mean(400*p3(j,:)'<0);
% infts(j,4,1) = mean(400*p5(j,:)'>4); infts(j,4,2)=mean(400*p5(j,:)'<0);
% infts(j,5,1) = mean(400*p7(j,:)'>4); infts(j,5,2)=mean(400*p7(j,:)'<0);
% infts(j,6,1) = mean(400*p10(j,:)'>4); infts(j,6,2)=mean(400*p10(j,:)'<0);
% 
% inftsa(j,1,1) = mean(400*pa1(j,:)'>4); inftsa(j,1,2)=mean(400*pa1(j,:)'<0);
% inftsa(j,2,1) = mean(400*pa2(j,:)'>4); inftsa(j,2,2)=mean(400*pa2(j,:)'<0);
% inftsa(j,3,1) = mean(400*pa3(j,:)'>4); inftsa(j,3,2)=mean(400*pa3(j,:)'<0);
% inftsa(j,4,1) = mean(400*pa5(j,:)'>4); inftsa(j,4,2)=mean(400*pa5(j,:)'<0);
% inftsa(j,5,1) = mean(400*pa7(j,:)'>4); inftsa(j,5,2)=mean(400*pa7(j,:)'<0);
% inftsa(j,6,1) = mean(400*pa10(j,:)'>4); inftsa(j,6,2)=mean(400*pa10(j,:)'<0);
% end 
% 
% 
% figure; 
% subplot(2,2,1); 
% plot(date,infL(:,1),'k',date,infLa(:,1),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average>4%)')
% subplot(2,2,2); 
% plot(date,infL(:,2),'k',date,infLa(:,2),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average>3%)')
% subplot(2,2,3); 
% plot(date,infL(:,3),'k',date,infLa(:,3),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average<1%)')
% subplot(2,2,4); 
% plot(date,infL(:,4),'k',date,infLa(:,4),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average<0%)')
% 
% figure;
% subplot(2,2,1); 
% plot(date,infS(:,1),'k',date,infSa(:,1),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average>4%)')
% subplot(2,2,2); 
% plot(date,infS(:,2),'k',date,infSa(:,2),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average>3%)')
% subplot(2,2,3); 
% plot(date,infS(:,3),'k',date,infSa(:,3),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average<1%)')
% subplot(2,2,4); 
% plot(date,infS(:,4),'k',date,infSa(:,4),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average<0%)')
% 
% figure;
% subplot(2,2,1); 
% plot(date,inf(:,1),'k',date,infa(:,1),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average>4%)')
% subplot(2,2,2); 
% plot(date,inf(:,2),'k',date,infa(:,2),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average>3%)')
% subplot(2,2,3); 
% plot(date,inf(:,3),'k',date,infa(:,3),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average<1%)')
% subplot(2,2,4); 
% plot(date,inf(:,4),'k',date,infa(:,4),'b--','LineWidth',2);
% grid on;
% xlabel('time')
% legend( 'No Resampling','Resampling')
% title('Prob(5yr Average<0%)')
% 
% figure;
% subplot(3,2,1); 
% plot(dates,ts_data(:,1),'k','LineWidth',2);
% grid on;
% xlabel('time')
% title('Long-run Inflation Expectations')
% 
% subplot(3,2,2); 
% plot(dates,ts_data(:,2),'k','LineWidth',2);
% grid on;
% xlabel('time')
% title('Short-run Inflation Expectations')
% 
% subplot(3,2,3); 
% plot(dates,ts_data(:,3),'k','LineWidth',2);
% grid on;
% xlabel('time')
% title('CPI Inflation')
% 
% subplot(3,2,4); 
% plot(dates,ts_data(:,4),'k','LineWidth',2);
% grid on;
% xlabel('time')
% title('Chicago Fed National Activity Index')
% 
% subplot(3,2,5); 
% plot(dates,ts_data(:,1),'k','LineWidth',2);
% grid on;
% xlabel('time')
% title('Wu-Xia Shadow Funds Rate')
% 
% sfile =['trend_forecast_altdata.mat']; 
% 
% 
% save(sfile,'Y50','Y75','Y25','YYY','YYYA','Y25A','Y50A','Y75A','pL','pS','p','infL','infS','inf','infLa','infSa','infa'); 

