%  This file generates long-run (10 year ahead) forecasts


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VAR, Core PCE Inflation

clear


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 
DFILE = 'posterior.mat'; 
%save(DFILE,'SD','QD','HD','CD','VD','LD','SS','INDEX_RW','SWT','SA','SQ','SH','SC','SV','SL','SSS'); 

%load(DFILE,'SD')    % time-varying parameters 
load(DFILE,'SA')    % time-varying parameters without resampling 
%load(DFILE,'QD')    % covariance matrix for innovations to TVPs 
load(DFILE,'SQ')    % covariance matrix for innovations to TVPs without resampling 
%load(DFILE,'HD')    % stochastic volatility 
load(DFILE,'SH')    % stochastic volatility without resampling 
%load(DFILE,'VD')    % standard deviations of innovations to log volatility  
load(DFILE,'SV')    % stadnadr deviations of innovations to log volatility without resampling  
%load(DFILE,'CD')    % loadings for measurement innovation variance  
load(DFILE,'SC')    % loadings for measurement innovation variance without resampling 
%load(DFILE,'LD')    % Cholesky factorization 
load(DFILE,'SL')    % Cholesky factorization without resampling 
%load(DFILE,'SS')    % Strucutral shocks 
load(DFILE,'SSS')   % Structural shocks without resampling 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


randn('state',100)

[NK,TP,P] = size(SA);
[NQ,NQ,P]= size(SQ);
[HT,NH,P]=size(SH);
[NV,P]=size(SV);
[NC,P]=size(SC);

%  FIRST LOAD THE DATA

ts		= importdata('data_with_labels.xlsx');    
ts_data	= ts.data;
dates	= 1970.25:0.25:2017.75;
dates 	= dates';

%% DEFINE VARIABLES
pi10e	= ts_data(:,1); % 400*LN(1+CPI LONG-TERM FORWARD EXP/400)	 
pi1e	= ts_data(:,2);	% 400*LN(1+CPI 1 YEAR AHEAD EXP/400)
pi	 	= ts_data(:,3); % 400*LN(1+CPI QUARTERLY PRECENT CHANGE AT AN ANNUALZIED RATE /400)
act  	= ts_data(:,4); % CHICAGO FED NATIONAL ACTIVITY INDEX (QUARTERLY AVERAGE OF MONTHLY VALUES)
int  	= ts_data(:,5); % 400*LN(1+FED FUNDS RATE/400) NOTE: FROM 2009:Q1-2015:Q4 WU-XIA SHADOW RATE IS USED

%% ORDER VARIABLES IN VAR (FOLLOWS FROM CLARK AND DAVIG'S IDENTIFICATION)
y 		= [pi10e, pi1e, pi, act, int]; % 
y         = y/400;
 
L = 1; % VAR lag order 
[T,N] = size(y);   % T: Total Sample Size (1970:Q2-2017:Q4), N: Number of variables (5) 

% create the quarterly indicator to distinguish different horizon length of long-term forward inflation forecasts 
% 35 (qindex=1), 34 (qindex=2) , 33 (qindex=3), 32 (qindex=4)  
qindex = 1+(dates-floor(dates))./0.25;

% adjust for lags, create rh variables
% lagged dependent variables: X1 = [y(t-1) y(t-1) ... y(t-L)]
X1 = zeros(T-L,1+(N*L));
X1(1:T-L,1) = ones(T-L,1);

for i = 1:L 
	X1(1:T-L,2+(N*(i-1)):1+(i*N)) = y(1+L-i:T-i,:);
end

Y = y(1+L:T,:)'; % Third row to the end, then transpose  : N by T-L
X = XMAT_5(X1',X1',X1',X1',X1');   % 
% partitioning the data
% initial estimates through T0
Train = 49+L; % 1970:Q2 - 1982:Q4 
T0 = Train-L; %adjust for training sample
Y0 = Y(:,1:T0);  
YS = Y(:,1+T0:T-L);
Y_aux = Y(:,T0-2:T-L)';
X0 = X(:,:,1:T0);
XS = X(:,:,1+T0:T-L);
X01 = X1(1:T0,:);
XS1 = X1(1+T0:T-L,:);
[SI,PI,RI] = SUR(Y0,X0,T0);   % PI, RI controls the tightness of priors for initial latent variables (coefficients, volatility) 
WW      =  var_weight(SI,PI,N,L);    
LFF     =  zeros(TP,P); 
lambda = 2; 
%for tt=1:T
for jj=1:P 
    LFF(:,jj) = im_weight(SA(:,:,jj),N,L,WW,qindex(end-137:end),lambda); 
end
% Select draws that have non-negligible probabilities

index = find(mean(LFF)>-76.084);  % 
lindex = length(index); 
if lindex>0.01*length(LFF)
LX =exp(mean(LFF(:,index)))./sum(exp(mean(LFF(:,index))));
INDEX_RW = randsample(1:lindex,P,true,LX'); 
 SD = SA(:,:,index(INDEX_RW)); 
 QD = SQ(:,:,index(INDEX_RW)); 
 HD = SH(:,:,index(INDEX_RW));
 CD = SC(:,index(INDEX_RW));
 VD = SV(:,index(INDEX_RW));
 LD = SL(:,:,index(INDEX_RW));
 SS = SSS(:,:,index(INDEX_RW));
%end
else 
disp('too few draws to resample: decrease lambda'); 
return; 
end 

date = 1983.5:0.25:2017.75; % sample runs from 1983Q1 to 2017Q4

FH         = 40;  % forecast horizon  
TH         = length(date);  % the recent 50 years 
SAF= zeros(NK,FH,P);     % without resampling 
SAFA = zeros(NK,FH,P);   % wiht resampling 
SHF= zeros(FH,NH,P);     % without resampling    
SHFA = zeros(FH,NH,P);   % with resampling  
YYF= zeros(N,FH,P);    % without resampling  
YYFA=zeros(N,FH,P);    % with resampling 
YYY = zeros(N,FH,TH,P); %  without resampling 
YYYA = zeros(N,FH,TH,P); % with resampling 
Y25  = zeros(N,TH);  
Y50  = zeros(N,TH); 
Y75  = zeros(N,TH); 
Y25A = zeros(N,TH); 
Y50A = zeros(N,TH); 
Y75A = zeros(N,TH); 
T=TH;
for jj=1:TH
  tic;
    
for i = 1:P   % parameter uncertainty 
   
    
    % Step 1: Innovations to time-varying coefficients 
      
    for tt=1:FH
    %    if tt==1   
      SAF(:,tt,i) = SA(:,T-TH+jj,i)+chol(SQ(:,:,i),'lower')*randn(NQ,1); 
      SAFA(:,tt,i) = SD(:,T-TH+jj,i)+chol(QD(:,:,i),'lower')*randn(NQ,1); 
     % SAF(:,tt,i)=SA(:,T-TH+jj,i);
     % SAFA(:,tt,i)=SD(:,T-TH+jj,i); 
        else 
      SAF(:,tt,i) = SAF(:,tt-1,i)+chol(SQ(:,:,i),'lower')*randn(NQ,1);
      SAFA(:,tt,i) =SAFA(:,tt-1,i)+chol(QD(:,:,i),'lower')*randn(NQ,1);   
    %      SAF(:,tt,i)=SAF(:,tt-1,i);
    %  SAFA(:,tt,i)=SAFA(:,tt-1,i); 
       
       end 
     end         
  
    
    % Step 2: Innovations to stochastic volatility 
    for tt=1:FH
        if tt==1  
    SHF(tt,:,i) = log(SH(T-TH+jj,:,i))+(diag(SV(:,i))*randn(NV,1))';   
    SHF(tt,:,i) = exp(SHF(tt,:,i)); 
    %SHF(tt,:,i) = SH(T-TH+jj,:,i); 
     SHFA(tt,:,i) = log(HD(T-TH+jj,:,i))+(diag(VD(:,i))*randn(NV,1))';
     SHFA(tt,:,i) = exp(SHFA(tt,:,i)); 
    %SHFA(tt,:,i) = HD(T-TH+jj,:,i);  
        else 
     SHF(tt,:,i) = log(SHF(tt-1,:,i))+(diag(SV(:,i))*randn(NV,1))';
     SHF(tt,:,i) = exp(SHF(tt,:,i));
     SHFA(tt,:,i) = log(SHFA(tt-1,:,i))+(diag(VD(:,i))*randn(NV,1))'; 
     SHFA(tt,:,i) = exp(SHFA(tt,:,i));  
        end 
    end 
    
    % Step 3: Innovations to equation errors 
    for tt=1:FH 
        innov=inv(SL(:,:,i))*diag(sqrt(SHF(tt,:,i)))*randn(N,1);
        innov2=inv(LD(:,:,i))*diag(sqrt(SHFA(tt,:,i)))*randn(N,1);  
        if tt==1 
            theta = reshape(SAF(:,tt,i),1+N*L,N)';
            theta_a = reshape(SAFA(:,tt,i),1+N*L,N)';
    YYF(:,tt,i) = theta(:,1)+theta(:,2:end)*[Y_aux(jj+3,:)';Y_aux(jj+2,:)';Y_aux(jj+1,:)';Y_aux(jj,:)'];
    YYFA(:,tt,i)= theta_a(:,1)+theta_a(:,2:end)*[Y_aux(jj+3,:)';Y_aux(jj+2,:)';Y_aux(jj+1,:)';Y_aux(jj,:)'];  
        elseif tt==2
            theta = reshape(SAF(:,tt,i),1+N*L,N)';
            theta_a = reshape(SAFA(:,tt,i),1+N*L,N)';
    YYF(:,tt,i) = theta(:,1)+theta(:,2:end)*[YYF(:,tt-1,i);Y_aux(jj+3,:)';Y_aux(jj+2,:)';Y_aux(jj+1,:)'];
    YYFA(:,tt,i)= theta_a(:,1)+theta_a(:,2:end)*[YYFA(:,tt-1,i);Y_aux(jj+3,:)';Y_aux(jj+2,:)';Y_aux(jj+1,:)'];  
        elseif tt==3 
            theta = reshape(SAF(:,tt,i),1+N*L,N)';
            theta_a = reshape(SAFA(:,tt,i),1+N*L,N)';
     YYF(:,tt,i) = theta(:,1)+theta(:,2:end)*[YYF(:,tt-1,i);YYF(:,tt-2,i);Y_aux(jj+3,:)';Y_aux(jj+2,:)'];
    YYFA(:,tt,i)= theta_a(:,1)+theta_a(:,2:end)*[YYFA(:,tt-1,i);YYFA(:,tt-2,i);Y_aux(jj+3,:)';Y_aux(jj+2,:)'];     
        elseif tt==L 
             theta = reshape(SAF(:,tt,i),1+N*L,N)';
            theta_a = reshape(SAFA(:,tt,i),1+N*L,N)';
       YYF(:,tt,i) = theta(:,1)+theta(:,2:end)*[YYF(:,tt-1,i);YYF(:,tt-2,i);YYF(:,tt-3,i);Y_aux(jj+3,:)'];
    YYFA(:,tt,i)= theta_a(:,1)+theta_a(:,2:end)*[YYFA(:,tt-1,i);YYFA(:,tt-2,i);YYFA(:,tt-3,i);Y_aux(jj+3,:)'];      
        else
             theta = reshape(SAF(:,tt,i),1+N*L,N)';
            theta_a = reshape(SAFA(:,tt,i),1+N*L,N)';
      YYF(:,tt,i) = theta(:,1)+theta(:,2:end)*[YYF(:,tt-1,i);YYF(:,tt-2,i);YYF(:,tt-3,i);YYF(:,tt-L,i)];
      YYFA(:,tt,i)= theta_a(:,1)+theta_a(:,2:end)*[YYFA(:,tt-1,i);YYFA(:,tt-2,i);YYFA(:,tt-3,i);YYFA(:,tt-L,i)];           
        end 
    end   
    YYY(:,:,jj,i) = YYF(:,:,i); 
    YYYA(:,:,jj,i) = YYFA(:,:,i); 
end
 for kk=1:N
 Y50(kk,jj) = quantile(mean(YYY(kk,21:40,jj,:),2),0.5);                                            % 10 year ahead forecast 
 Y25(kk,jj) = quantile(mean(YYY(kk,21:40,jj,:),2),0.25);
 Y75(kk,jj) = quantile(mean(YYY(kk,21:40,jj,:),2),0.75);
 Y50A(kk,jj)= quantile(mean(YYYA(kk,21:40,jj,:),2),0.5); 
 Y25A(kk,jj)= quantile(mean(YYYA(kk,21:40,jj,:),2),0.25); 
 Y75A(kk,jj)= quantile(mean(YYYA(kk,21:40,jj,:),2),0.75); 
 end
 toc; 
end
sfile =['trend_forecast_original.mat']; 


save(sfile,'Y50','Y75','Y25','YYY','YYYA','Y25A','Y50A','Y75A','LX'); 


for kk=1:5
  if kk==4
     kk=kk+1;
  end 
figure;
subplot(2,1,1); 
    fill([date fliplr(date)], 400*[Y75(kk,:) fliplr(Y25(kk,:))],[0.5 0.5 0.5]); 
    hold on; 
plot(date,400*Y50(kk,:),'k',date,400*Y25(kk,:),'b--',date,400*Y75(kk,:),'b--','LineWidth',2);    
grid on; 
hold off; 
 ylabel('% percent')
legend('IQR','Median')
xlim([date(1) date(end)])
subplot(2,1,2); 
 fill([date fliplr(date)], 400*[Y75A(kk,:) fliplr(Y25A(kk,:))],[0.5 0.5 0.5]); 
    hold on; 
plot(date,400*Y50A(kk,:),'k',date,400*Y25A(kk,:),'b--',date,400*Y75A(kk,:),'b--','LineWidth',2);    
grid on; 
hold off; 
  ylabel('% percent')
legend('IQR: with resampling','Median: with resampling')
xlim([date(1) date(end)])
if kk==1 title('Long-run Inflation Expectations')
elseif kk==2 title ('Short-run Inflation Expectations')
elseif kk==3 title ('CPI Inflation')
elseif kk==5 title ('Interest Rate')
 end
end 

% Inflation Tail Risk 

pL  = squeeze(mean(YYY(1,1:20,:,:),2));
pS  = squeeze(mean(YYY(2,1:20,:,:),2)); 
p   = squeeze(mean(YYY(3,1:20,:,:),2));
pLa  = squeeze(mean(YYYA(1,1:20,:,:),2));
pSa  = squeeze(mean(YYYA(2,1:20,:,:),2)); 
pa   = squeeze(mean(YYYA(3,1:20,:,:),2));

infL  = zeros(TH,4); 
infS  = zeros(TH,4);  
inf   = zeros(TH,4);

infLa = zeros(TH,4); 
infSa = zeros(TH,4); 
infa  = zeros(TH,4); 
for j=1:TH
infL(j,1) = mean(400*pL(j,:)'>4); infL(j,2) = mean(400*pL(j,:)'>3); infL(j,3) = mean(400*pL(j,:)'<1); infL(j,4) = mean(400*pL(j,:)'<0); 
infS(j,1) = mean(400*pS(j,:)'>4); infS(j,2) = mean(400*pS(j,:)'>3); infS(j,3) = mean(400*pS(j,:)'<1); infS(j,4) = mean(400*pS(j,:)'<0); 
inf(j,1)  = mean(400*p(j,:)'>4);  inf(j,2)  = mean(400*p(j,:)'>3);  inf(j,3)  = mean(400*p(j,:)'<1);  inf(j,4)  = mean(400*p(j,:)'<0); 
infLa(j,1) = mean(400*pLa(j,:)'>4); infLa(j,2) = mean(400*pLa(j,:)'>3); infLa(j,3) = mean(400*pLa(j,:)'<1); infLa(j,4) = mean(400*pLa(j,:)'<0); 
infSa(j,1) = mean(400*pSa(j,:)'>4); infSa(j,2) = mean(400*pSa(j,:)'>3); infSa(j,3) = mean(400*pSa(j,:)'<1); infSa(j,4) = mean(400*pSa(j,:)'<0); 
infa(j,1)  = mean(400*pa(j,:)'>4);  infa(j,2)  = mean(400*pa(j,:)'>3);  infa(j,3)  = mean(400*pa(j,:)'<1);  infa(j,4)  = mean(400*pa(j,:)'<0); 
end 

date=date';
figure; 
subplot(2,2,1); 
plot(date,infL(:,1),'k',date,infLa(:,1),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average>4%)')
subplot(2,2,2); 
plot(date,infL(:,2),'k',date,infLa(:,2),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average>3%)')
subplot(2,2,3); 
plot(date,infL(:,3),'k',date,infLa(:,3),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average<1%)')
subplot(2,2,4); 
plot(date,infL(:,4),'k',date,infLa(:,4),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average<0%)')

figure;
subplot(2,2,1); 
plot(date,infS(:,1),'k',date,infSa(:,1),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average>4%)')
subplot(2,2,2); 
plot(date,infS(:,2),'k',date,infSa(:,2),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average>3%)')
subplot(2,2,3); 
plot(date,infS(:,3),'k',date,infSa(:,3),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average<1%)')
subplot(2,2,4); 
plot(date,infS(:,4),'k',date,infSa(:,4),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average<0%)')

figure;
subplot(2,2,1); 
plot(date,inf(:,1),'k',date,infa(:,1),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average>4%)')
subplot(2,2,2); 
plot(date,inf(:,2),'k',date,infa(:,2),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average>3%)')
subplot(2,2,3); 
plot(date,inf(:,3),'k',date,infa(:,3),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average<1%)')
subplot(2,2,4); 
plot(date,inf(:,4),'k',date,infa(:,4),'b--','LineWidth',2);
grid on;
xlabel('time')
legend( 'No Resampling','Resampling')
title('Prob(5yr Average<0%)')









sfile =['trend_forecast_alt.mat']; 


save(sfile,'Y50','Y75','Y25','YYY','YYYA','Y25A','Y50A','Y75A','pL','pS','p','infL','infS','inf','LX'); 

