% The code modifies Cogley-Sargent (2005) TVP-VAR model to impose
% cross-equation restrictions using importance sampling weights. 

%  This simulates a var(2) for long-term and short-term inflation expetcations, headline CPI inflation, economic activity index, and the federal funds rate including the shadow rate during the ZLB period.  
%  There is stochastic volatility in the measurement innovation, but state 
%  innovations are homoskedastic. 

%  modified by Taeyoung Doh: 03/13/2018 
%  modified by Lee Smith: 03/21/2018
%  linked files  : add the list of files used inside the troutine 

clear
tic;

NG = 5000; % number of draws from Gibbs sampler per data file %5000
D  = 10; % sampling interval from markov chain (memory constraint, mixing) %20

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% catalog of data files  : Base prior under lamdbda 
DFILE(1,:) =  ['var101base_no_stab.mat'];
DFILE(2,:) =  ['var102base_no_stab.mat'];
DFILE(3,:) =  ['var103base_no_stab.mat'];
DFILE(4,:) =  ['var104base_no_stab.mat'];
DFILE(5,:) =  ['var105base_no_stab.mat'];
DFILE(6,:) =  ['var106base_no_stab.mat'];
DFILE(7,:) =  ['var107base_no_stab.mat'];
DFILE(8,:) =  ['var108base_no_stab.mat'];
DFILE(9,:) =  ['var109base_no_stab.mat'];
DFILE(10,:) = ['var110base_no_stab.mat'];
DFILE(11,:) = ['var111base_no_stab.mat'];
DFILE(12,:) = ['var112base_no_stab.mat'];
DFILE(13,:) = ['var113base_no_stab.mat'];
DFILE(14,:) = ['var114base_no_stab.mat'];
DFILE(15,:) = ['var115base_no_stab.mat'];
DFILE(16,:) = ['var116base_no_stab.mat'];
DFILE(17,:) = ['var117base_no_stab.mat'];
DFILE(18,:) = ['var118base_no_stab.mat'];
DFILE(19,:) = ['var119base_no_stab.mat'];
DFILE(20,:) = ['var120base_no_stab.mat'];
NF = size(DFILE,1);

% variables to be saved
varname(1,:)  = ['SD'];   % states : time-varying coefficients (re-weighted)
varname(2,:)  = ['SA'];   % states : time-varying coefficients (unweighted)
varname(3,:)  = ['QD'];   % variance of state innovation
varname(4,:)  = ['HD'];   % svol for measurement innovation
varname(5,:)  = ['CD'];   % loadings for measurement innovation variance
varname(6,:)  = ['VD'];   % standard deviation of volatility innovations
varname(7,:)  = ['NP'];   % number of not positive definite matrices in Gibbs1 fortran
varname(8,:)  = ['EP'];   % size of epsilon necessary to render the matrix positive definite
varname(9,:)  = ['LD'];   % Cholesky factorization (Lower-Triangular)
varname(10,:) = ['SS'];   % structural shocks
varname(11,:) = ['LF'];   % Loss function for expectatioanl consistency (T x draws)
varname(12,:) = ['WT'];   % Importance sampling weights for SA (draws x 1)
%varname(13,:) = ['IR'];  % Index of draws of SA to sample after re-weighting (draws x 1)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% FIRST LOAD THE DATA
ts		= importdata('data_with_labels.xlsx');    
ts_data	= ts.data;
dates	= 1970.25:0.25:2017.75;
dates 	= dates';

%% DEFINE VARIABLES
pi10e	= ts_data(:,1); % 400*LN(1+CPI LONG-TERM FORWARD EXP/400)	 
pi1e	= ts_data(:,2);	% 400*LN(1+CPI 1 YEAR AHEAD EXP/400)
pi	 	= ts_data(:,3); % 400*LN(1+CPI QUARTERLY PRECENT CHANGE AT AN ANNUALZIED RATE /400)
act  	= ts_data(:,4); % CHICAGO FED NATIONAL ACTIVITY INDEX (QUARTERLY AVERAGE OF MONTHLY VALUES)
int  	= ts_data(:,5); % 400*LN(1+FED FUNDS RATE/400) NOTE: FROM 2009:Q1-2015:Q4 WU-XIA SHADOW RATE IS USED

%% ORDER VARIABLES IN VAR (FOLLOWS FROM CLARK AND DAVIG'S IDENTIFICATION)
y 		= [pi10e, pi1e, pi, act, int]; % 
y         = y/400;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L = 4; % VAR lag order 
[T,N] = size(y);   % T: Total Sample Size (1970:Q2-2017:Q4), N: Number of variables (5) 

% create the quarterly indicator to distinguish different horizon length of long-term forward inflation forecasts 
% 35 (qindex=1), 34 (qindex=2) , 33 (qindex=3), 32 (qindex=4)  
qindex = 1+(dates-floor(dates))./0.25;

% adjust for lags, create rh variables
% lagged dependent variables: X1 = [y(t-1) y(t-1) ... y(t-L)]
X1 = zeros(T-L,1+(N*L));
X1(1:T-L,1) = ones(T-L,1);

for i = 1:L 
	X1(1:T-L,2+(N*(i-1)):1+(i*N)) = y(1+L-i:T-i,:);
end

Y = y(1+L:T,:)'; % Third row to the end, then transpose  : N by T-L
X = XMAT_5(X1',X1',X1',X1',X1');   % 

% partitioning the data
% initial estimates through T0
Train = 49+L; % 1970:Q2 - 1982:Q4 
T0 = Train-L; %adjust for training sample
Y0 = Y(:,1:T0);  
YS = Y(:,1+T0:T-L);
X0 = X(:,:,1:T0);
XS = X(:,:,1+T0:T-L);
X01 = X1(1:T0,:);
XS1 = X1(1+T0:T-L,:);
date = 1983.0 + [0:1:size(YS,2)-1]'; % sample runs from 1983Q1 to 2017Q4

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% an informative prior: SUR estimates through T0

[SI,PI,RI] = SUR(Y0,X0,T0);   % PI, RI controls the tightness of priors for initial latent variables (coefficients, volatility) 
%PI = PI; 
df = N*(1+N*L) + 1; % prior degrees of freedom  : controls the tightness of priors for the time-variation in coefficients 
DF = df + T - L - T0; % posterior degrees of freedom 
% (cogley-sargent) : control implied variation in trend
% implied prior mean and AR roots
S = [eye(N*(L-1)),zeros(N*(L-1),N)];
ASI = reshape(SI,1+N*L,N);
A = [ASI(2:end,:)'; S];
rr = max(abs(eig(A)));
A0 = [ASI(1,:)' ; zeros(N*(L-1),1)];
M = inv(eye(N*L) - A)*A0;

% Calibration of the shrinkage parameter for Q0 : (added by Taeyoung Doh) 
SAA = SI + real(sqrtm(PI))*ones(N*(1+N*L),1);   % One stadard deviation shocks for all the coefficients 
AA = [SAA(2:1+N*L,1)'; SAA(2+(1+N*L):2*(1+N*L),1)'; SAA(2+2*(1+N*L):3*(1+N*L),1)';SAA(2+3*(1+N*L):4*(1+N*L))';SAA(2+4*(1+N*L):5*(1+N*L))';S];
AA0 =  [SAA(1,1); SAA(2+N*L,1); SAA(3+2*N*L,1);SAA(4+3*N*L,1);SAA(5+4*N*L,1); zeros(N*(L-1),1)]; 
MA = inv(eye(N*L) - AA)*AA0;
dMA = MA-M; 
adj = min(diag(cov(Y0'))./(T0*(dMA(1:N,1).^2)));
Q0 = adj*PI;   % The variance of the change in the trend during T0 period is less than or equal to the variances of observed variables during the pre-sample period. 
% Calibration over! 

TQ0 = df*Q0; 
TQ  = TQ0; 

%%
% YLS = Y(:,1:45)';
% [Tbig,Nbig] = size(YLS);
% Tbig = Tbig;
% Nbigcomp = N*L;
% vlag   = L;
% BIG_X 		= zeros(Tbig-vlag,Nbig*vlag+1);
% BIG_X(:,1) 	= ones(Tbig-vlag,1);
% for lags = 1:1:vlag;
	% lower_index_row		= vlag+1-lags;
	% upper_index_row		= Tbig-lags;
	% upper_index_col		= Nbig*lags+1;
	% lower_index_col		= upper_index_col - Nbig + 1;
	% BIG_X(:,lower_index_col:upper_index_col) = YLS(lower_index_row:upper_index_row,:); 	
% end;

% BIG_Y = YLS(vlag+1:Tbig,:);

% BIG_A 			= (BIG_X'*BIG_X)\BIG_X'*BIG_Y; 
%%

'prior mean'; M
'prior AR roots'; rr
clear M  rr

% clear initial sample
clear y Y X Y0 X0 X01 T0 T1
[T,N] = size(YS');

% priors for SVOL parameters
% alpha, delta (normal) (same for all stochastic volatilities) (not needed for random walk spec)
b0 = [0,1]';
k = size(b0,1);
Sb0 = 10000*eye(k,k);
b = zeros(2,N); % alpha, delta in log volatility regression

% priors for sv (inverse gamma) (standard dev for volatility innovation)
sv0 = .002;
v0 = 1;
d0 = sv0^2;
eh = zeros(T,N); % volatility innovations

% priors for log h0 and correlation parameters (normal, ballpark numbers)
ss0 = 10;                             
mu0 = log(diag(RI));

% prior for correlation parameters for stochastic volatilities (normal)
ch0 = zeros(N*(N-1)/2,1);
ch = ch0;
ssc0 = 10000*eye(N*(N-1)/2,N*(N-1)/2);
CF = CHOFAC(N,ch0);  

% hyperparameter for the tightness of consistency requirements 
lambda = 1; 
W      =  var_weight(SI,PI,N,L);                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialize gibbs arrays
SA = zeros(N*(1+N*L),T,NG); % draws of the state vector
QA = zeros(N*(1+N*L),N*(1+N*L),NG); % draws of covariance matrix for state innovations
SV = zeros(N,NG); % standard error for volatility innovation
H = zeros(T+1,N,NG); % stochastic volatilities 
CH = zeros(N*(N-1)/2,NG);
e_shocks 	= zeros(T,N,NG); % reduced form residuals (added by Lee Smith)
CholFac  	= zeros(N,N,NG); % Cholesky Factorization (added by Lee Smith) 
eps_shocks  = zeros(T,N,NG); % structural shocks (added by Lee Smith)

LF 			= -1*ones(T,NG)*1e12;   % Loss function for expectatioanl consistency (T x draws)
WEIGHT  	= zeros(NG,1);   		% Importance sampling weights for SA (draws x 1)
INDEX_RW	= zeros(NG,1);			% Index of draws of SA to sample after re-weighting (draws x 1)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initial for log(h(t))
lh(1:2,:) = ones(2,1)*mu0';
dy = diff(YS');
e = dy - ones(T-1,1)*mean(dy);
lh(3:T+1,:) = log(e.^2);
H0 = exp(lh);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% warm up for stochastic volatilities; shut down var, treat dy as var innovation

evar = [zeros(N,1) e'];
H(:,:,1) = H0;
for iter = 2:NG
	%R conditional on states and data (svol programs)
	lh = log(H(:,:,iter-1));
	for i = 1:N
		% sv|b,lh,ch,y
		eh(:,i) = lh(2:T+1,i) - lh(1:T,i);  % random walk
		v = IG2(v0,d0,eh(:,i));
		SV(i,iter) = v^.5;
	end

	% orthogonalize var innovations
	%evar = INNOVM(YS,XS1,SA(:,:,iter),N,T,L);
	CF = CHOFAC(N,ch);  
	f = (CF*evar)';          
    
	% lh|ch,sv,b1,y
	for i = 1:N   
		H(1,i,iter) = svmh0(H(2,i,iter-1),0,1,SV(i,iter),mu0(i,1),ss0);
		for t = 2:T 
			H(t,i,iter) = svmh(H(t+1,i,iter-1),H(t-1,i,iter),0,1,SV(i,iter),f(t-1,i),H(t,i,iter-1)); 
		end
		H(T+1,i,iter) = svmhT(H(T,i,iter),0,1,SV(i,iter),f(T,i),H(T+1,i,iter-1)); 
	end
        
	% ch|sv,b,lh,y
	k = 0;
	for i = 2:N 
		lhs = H(2:T+1,i,iter).^.5;  
		for n = 1:N
			yhs(:,n) = (evar(n,:)')./lhs;
		end 
		yr = yhs(:,i);
		xr = -yhs(:,1:i-1);
		j = k+1;
		k = i-1+k;
		ch(j:k,1) = BAYESREG(ch0(j:k),ssc0(j:k,j:k),1,yr,xr);
	end
    CH(:,iter) = ch;  
end  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initial draw of states
H0 = H(:,:,NG);
[S0,P0,P1] = kfR(YS,XS,Q0,CF,H0,SI,PI,T,N,L); 


mlmax = 2;
count_expl = 0;
%while mlmax >= 1    % Draw a stationary process 
	SA1 = GIBBS1(S0,P0,P1,T,N,L);     
%	for j = 1:T,
%		A = [SA1(2:1+N*L,j)'; SA1(2+(1+N*L):2*(1+N*L),j)'; SA1(2+2*(1+N*L):3*(1+N*L),j)';SA1(2+3*(1+N*L):4*(1+N*L),j)';SA1(2+4*(1+N*L):5*(1+N*L),j)';S];
%		lmax(j,1) = max(abs(eig(A)))';
%	end 
%	count_expl = count_expl + 1;
%	mlmax = max(lmax);	
%end

SA(:,:,1) = SA1;
H(:,:,1) = H(:,:,NG);
CH(:,1) = CH(:,NG);
SV(:,1) = SV(:,NG);

% compute shocks and cholesky factorization for initial storage (added by Lee Smith)
e_shocks(:,:,1) = INNOVM(YS,XS1,SA(:,:,1),N,T,L)';
CholFac(:,:,1) = CHOFAC(N,CH(:,1));  
eps_shocks(:,:,1) = (CholFac(:,:,1)*(e_shocks(:,:,1)'))'; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% begin MCMC
count_unstable = 0;
notposdef=zeros(NF,NG);
work = NF*NG + 1;
appendmode=1; 

if appendmode==1
	exist 'append.mat';
	if ans==1
	ssfile = ['append.mat'];
	load(ssfile); 
	SA  = SA_append; 
	QA  = QA_append; 
	H   = H_append;
	CH   = CH_append;
	SV = SV_append;
	CholFac = LD_append;
	eps_shocks  = SS_append;
	LF  = LF_append;
	WEIGHT = WT_append;
    else
    filea =1;
    itera =2; 
    end
    
end 

for file = 1:NF,
	iter = 2; 
	notposdef=zeros(NG,T);
	epsilonvec=zeros(NG,T);

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	T1=int32(T);
	N1=int32(N);
	L1=int32(L);

	while iter <= NG   	
		% Print workbar
		progress = (file-1)*NG + iter;
		workbar(progress/work,'Performing MCMC','Progress');
	 
		if file == NF && iter == NG
			workbar(1,'Performing MCMC','Progress');
		end;
		
		% Draw Q conditional on states and data
		[TQ,DF] = iwpQ(SA(:,:,iter-1),N,T,L,TQ0,df);
		QA(:,:,iter) = gibbs2Q(TQ,DF,N,L);
		   
		% R conditional on states and data (svol programs)
		lh = log(H(:,:,iter-1));
		
		for i = 1:N,
			% sv|b,lh,ch,y
			eh(:,i) = lh(2:T+1,i) - lh(1:T,i);  % random walk
			v = IG2(v0,d0,eh(:,i));
			SV(i,iter) = v^.5;
		end
		
		% orthogonalize var innovations
		evar = INNOVM(YS,XS1,SA(:,:,iter-1),N,T,L);
		CF = CHOFAC(N,CH(:,iter-1));  
		f = (CF*evar)';           
		 
		% lh|ch,sv,b1,y
		for i = 1:N,   
		  H(1,i,iter) = svmh0(H(2,i,iter-1),0,1,SV(i,iter),mu0(i,1),ss0);
		  for t = 2:T, 
			 H(t,i,iter) = svmh(H(t+1,i,iter-1),H(t-1,i,iter),0,1,SV(i,iter),f(t-1,i),H(t,i,iter-1)); 
		  end
		  H(T+1,i,iter) = svmhT(H(T,i,iter),0,1,SV(i,iter),f(T,i),H(T+1,i,iter-1)); 
		end
		   
		% ch|sv,b,lh,y
		k = 0;
		for i = 2:N, 
		   lhs = H(2:T+1,i,iter).^.5;  
		   for n = 1:N,
			  yhs(:,n) = (evar(n,:)')./lhs;
		   end 
		   yr = yhs(:,i);
		   xr = -yhs(:,1:i-1);
		   j = k+1;
		   k = i-1+k;
		   ch(j:k,1) = BAYESREG(ch0(j:k),ssc0(j:k,j:k),1,yr,xr);
		end
		CH(:,iter) = ch;  
		CF = CHOFAC(N,CH(:,iter));  
			
		% states conditional on hyperparameters and data
		[S0,P0,P1] = kfR(YS,XS,QA(:,:,iter),CF,H(:,:,iter),SI,PI,T1,N1,L1); 
		[SA(:,:,iter) notposdef(iter,:) epsilonvec(iter,:)] = GIBBS1(S0,P0,P1,T1,N1,L1);
		%disp(max(epsilonvec(iter,:)))
		 
		% compute shocks and cholesky factorization for final storage (added by Lee Smith)
		e_shocks(:,:,iter) = INNOVM(YS,XS1,SA(:,:,iter),N,T,L)';
		CholFac(:,:,iter) = CF;  
		eps_shocks(:,:,iter) = (CF*(e_shocks(:,:,iter)'))'; 
		 
		% check stability; reject unstable draws
% 		stationary = 0;
% 		count_unstable = 0;
% 		while stationary==0  & count_unstable<5000 
% 			for j = 1:T;
% 				ASA		 = reshape(SA(:,j,iter),1+N*L,N);
% 				if L == 1
% 					A	 = ASA(2:end,:)';
% 				else
% 					A	 = [ASA(2:end,:)'; eye(N*(L-1)), zeros(N*(L-1),N)];
% 				end
% 				lmax(j,1) = max(abs(eig(A)))';
% 			end;
			
		%	ssfile = ['append.mat'];
		   
% 			 if max(lmax) <= 1,
% 	
				LF(:,iter) = im_weight(SA(:,:,iter),N,L,W,qindex,lambda);
				WEIGHT(iter) = exp(sum(LF(:,iter))./(1e5)); 
				if mod(iter,50)==0
				[file,iter]
                end 
%				SA_append=SA;
% 				QA_append=QA;
% 				H_append=H;
% 				CH_append=CH;
% 				SV_append=SV;
% 				LD_append=CholFac;
% 				SS_append=eps_shocks;
% 				LF_append=LF;
% 				WT_append=WEIGHT;
			  
				iter = iter+1;
% 				stationary = 1; 
%                 filea=file;
%                 itera=iter; 
% 				save(ssfile,'SA_append','QA_append','H_append','CH_append','SV_append','notposdef','epsilonvec','LD_append','SS_append','LF_append','WT_append','itera','filea');
% 			    if iter==3 || mod(iter,20)==0 
%                     [file,iter]
%                 end 
% 			elseif (max(lmax) > 1)  & count_unstable<5000
% 				count_unstable = count_unstable + 1;
%                 if mod(count_unstable,500)==0
%                 disp('States unstable'); 
%                 count_unstable
%                 [file,iter]
%                 end 
% 			else 
% 				count_unstable = 0; 
%                 stationary = 2; 
%                 iter = iter-1; 
% 				disp('States unstable, try again: 5000draws'); 
%             end     
     %   end
   end
   
    % Resample time-varying parameter coefficients 
%INDEX_RW = randsample(1:NG,NG,true,WEIGHT); 
% SD = SA(:,:,INDEX_RW); 
% QD = QA(:,:,INDEX_RW); 
% HD = H(:,:,INDEX_RW);
% CD = CH(:,INDEX_RW);
% VD = SV(:,INDEX_RW);
% LD = CholFac(:,:,INDEX_RW);
% SS = eps_shocks(:,:,INDEX_RW);
% sample from markov chain 

SD = SA(:,:,D:D:NG);
QD = QA(:,:,D:D:NG);
HD = H(:,:,D:D:NG);
CD = CH(:,D:D:NG);
VD = SV(:,D:D:NG);
NP = notposdef;
EP = epsilonvec;
LD = CholFac(:,:,D:D:NG);
SS = eps_shocks(:,:,D:D:NG);
WT = WEIGHT(D:D:NG);
%IR = INDEX_RW;
	   
save(DFILE(file,:),varname(1,:),varname(2,:),varname(3,:),varname(4,:),varname(5,:),varname(6,:),...
				   varname(7,:),varname(8,:),varname(9,:),varname(10,:),varname(11,:),...
				   varname(12,:));
       
% reinitialize gibbs arrays (buffer for back step)
SA(:,:,1) 			= SA(:,:,NG); 
QA(:,:,1) 			= QA(:,:,NG); 
H(:,:,1) 			= H(:,:,NG);
CH(:,1) 			= CH(:,NG);
SV(:,1) 			= SV(:,NG);
CholFac(:,:,1) 		= CholFac(:,:,NG);
eps_shocks(:,:,1) 	= eps_shocks(:,:,NG);
LF(:,1)				= LF(:,NG);
WEIGHT(1)			= WEIGHT(NG);  
    
end
%
toc;

% Resampling the posterior draws after burn-in 

NB = 10; % burn-in 50K draws. 
DFILE = 'var111base_no_stab.mat'; 
load(DFILE,'SD')    % time-varying parameters
load(DFILE,'QD')    % covariance matrix for innovations to TVPs 
load(DFILE,'HD')    % stochastic volatility 
load(DFILE,'VD')    % standard deviations of innovations to log volatility  
load(DFILE,'CD')    % Cholesky matrix for innovations to equation errors; Take the inverse of this matrix and multiply this by stochastic vol
load(DFILE,'LD')
load(DFILE,'SS')
load(DFILE,'LF')
[NK,T,P] = size(SD);
[NQ,NQ,P]= size(QD);
[HT,NH,P]=size(HD);
[NV,P]=size(VD);
[NC,P]=size(CD);
[NL1,NL2,P]=size(LD);
[NS1,NS2,P]=size(SS); 
P=length(WT); 
SA = zeros(NK,T,(NF-NB)*P);
SA(:,:,1:P) = SD;
SQ = zeros(NQ,NQ,(NF-NB)*P);
SQ(:,:,1:P) = QD;
SH = zeros(HT,NH,(NF-NB)*P);
SH(:,:,1:P)=HD;
SV = zeros(NV,(NF-NB)*P);
SV(:,1:P)=VD;
SC = zeros(NC,(NF-NB)*P); 
SC(:,1:P)=CD;
SL = zeros(NL1,NL2,(NF-NB)*P); 
SL(:,:,1:P)=LD;
SSS = zeros(NS1,NS2,(NF-NB)*P);
SSS(:,:,1:P)=SS;
SWT = zeros((NF-NB)*P,1);
SWT(1:P)=sum(LF(:,D:D:NG))'; 

                
for i = NB+2:NF,
    j = i - NB; 
    DFILE = ['var1' num2str(i,'%02.0f')  'base_no_stab.mat']; 
    load(DFILE,'SD')
    load(DFILE,'QD')   
    load(DFILE,'HD')    
    load(DFILE,'VD')    
    load(DFILE,'CD')     
    load(DFILE,'LD')
    load(DFILE,'SS')
    load(DFILE,'LF')
    SA(:,:,(j-1)*P+1:j*P) = SD;
    SQ(:,:,(j-1)*P+1:j*P) = QD;
    SH(:,:,(j-1)*P+1:j*P) = HD;
    SV(:,(j-1)*P+1:j*P)   = VD;
    SC(:,(j-1)*P+1:j*P)   = CD;
    SL(:,:,(j-1)*P+1:j*P) = LD; 
    SSS(:,:,(j-1)*P+1:j*P) = SS; 
    SWT((j-1)*P+1:j*P)=sum(LF(:,D:D:NG))'; 
clear SD QD HD VD CD LD SS LF
end
lambda2=0.02;
SWT = exp(lambda2*SWT);

INDEX_RW = randsample(1:(NF-NB)*P,(NF-NB)*P,true,SWT); 
SD = SA(:,:,INDEX_RW); 
 QD = SQ(:,:,INDEX_RW); 
 HD = SH(:,:,INDEX_RW);
 CD = SC(:,INDEX_RW);
 VD = SV(:,INDEX_RW);
 LD = SL(:,:,INDEX_RW);
 SS = SSS(:,:,INDEX_RW);

DDfile = ['posterior.mat']; 
save(DDfile,'SD','QD','HD','CD','VD','LD','SS','INDEX_RW','SWT','SA','SQ','SH','SC','SV','SL','SSS'); 


